
from vsg import token

from vsg.rules import token_indent

lTokens = []
lTokens.append(token.port_clause.close_parenthesis)


class rule_015(token_indent):
    '''
    This rule checks the indent of the closing parenthesis for port maps.

    **Violation**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
         );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
       );
    '''

    def __init__(self):
        token_indent.__init__(self, 'port', '015', lTokens)




from vsg.token import if_statement as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import condition
from vsg.vhdlFile.classify import sequence_of_statements


def detect(iToken, lObjects):
    '''
    if_statement ::=
        [ if_label : ]
            if condition then
                sequence_of_statements
            { elsif condition then
                sequence_of_statements }
            [ else
                sequence_of_statements ]
            end if [ if_label ] ;
    '''

    if utils.keyword_found('if', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.if_label, token.label_colon)
    iCurrent = utils.assign_next_token_required('if', token.if_keyword, iCurrent, lObjects)
    iCurrent = condition.classify_until(['then'], iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('then', token.then_keyword, iCurrent, lObjects)

    iCurrent = sequence_of_statements.detect(iCurrent, lObjects)

    while utils.is_next_token_one_of(['else', 'elsif'], iCurrent, lObjects):
        if utils.is_next_token('elsif', iCurrent, lObjects):
            iCurrent = utils.assign_next_token_required('elsif', token.elsif_keyword, iCurrent, lObjects)
            iCurrent = condition.classify_until(['then'], iCurrent, lObjects)
            iCurrent = utils.assign_next_token_required('then', token.then_keyword, iCurrent, lObjects)
            iCurrent = sequence_of_statements.detect(iCurrent, lObjects)
        else:
            iCurrent = utils.assign_next_token_required('else', token.else_keyword, iCurrent, lObjects)
            iCurrent = sequence_of_statements.detect(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('if', token.end_if_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_if_label, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)
    return iCurrent




from vsg.token import subprogram_kind as token

from vsg.vhdlFile import utils


def detect(iToken, lObjects):
    '''
    subprogram_kind ::=
        procedure | function
    '''

    if utils.is_next_token('procedure', iToken, lObjects):
        return True
    if utils.is_next_token('function', iToken, lObjects):
        return True
    return False


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_if('procedure', token.procedure_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if('function', token.function_keyword, iToken, lObjects)

    return iCurrent




from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_n_tokens_before_and_after_tokens_bounded_by_tokens(iToken, lTokens, lBetween, lAllTokens, oTokenMap):
    lReturn = []
    lIndexes = []

    lIndexes = utils.filter_tokens_between_tokens(lTokens, lBetween[0], lBetween[1], oTokenMap)

    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        lReturn.append(tokens.New(iIndex - iToken, iLine, lAllTokens[iIndex - iToken:iIndex + iToken + 1]))

    return lReturn





from vsg import parser
from vsg import violation

from vsg.rule_group import whitespace
from vsg.rules import utils as rules_utils

from vsg.vhdlFile import utils


class Rule(whitespace.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    left_token : token object
       The left token to check for a space between the right token

    right_token : token object
       The right token to check for a space between the left token
    '''

    def __init__(self, name, identifier):
        whitespace.Rule.__init__(self, name=name, identifier=identifier)
        self.left_token = None
        self.right_token = None
        self.number_of_spaces = 1
        self.configuration.append('number_of_spaces')

    def _get_tokens_of_interest(self, oFile):
        lToi_a = oFile.get_sequence_of_tokens_matching([self.left_token, parser.whitespace, self.right_token])
        lToi_b = oFile.get_sequence_of_tokens_matching([self.left_token, self.right_token])

        return utils.combine_two_token_class_lists(lToi_a, lToi_b)

    def _analyze(self, lToi):
        for oToi in lToi:
            if whitespace_token_exists(oToi):
                self.analyze_whitespace_token(oToi)
            else:
                self.analyze_no_whitespace_token(oToi)

    def analyze_no_whitespace_token(self, oToi):
        if self.number_of_spaces != 0:
            iSpaces = self.extract_expected_number_of_spaces()
            self.create_violation(oToi, iSpaces)

    def extract_expected_number_of_spaces(self):
        if self.number_of_spaces_is_an_integer():
            return self.number_of_spaces
        elif self.number_of_spaces_is_gte():
            return int(self.number_of_spaces[2:])
        elif self.number_of_spaces_is_gt():
            return int(self.number_of_spaces[1:])

    def analyze_whitespace_token(self, oToi):
        if self.number_of_spaces_is_an_integer():
            self.analyze_integer_spaces(oToi)
        elif self.number_of_spaces_is_gte():
            self.analyze_gte_spaces(oToi)
        elif self.number_of_spaces_is_gt():
            self.analyze_gt_spaces(oToi)
        elif self.number_of_spaces_is_plus():
            self.analyze_plus_spaces(oToi)

    def number_of_spaces_is_gt(self):
        if self.number_of_spaces.startswith('>'):
            return True
        return False

    def analyze_gt_spaces(self, oToi):
        iSpaces = int(self.number_of_spaces[1:]) + 1
        iWhitespaces = extract_length_of_whitespace(oToi)
        if iWhitespaces < iSpaces:
            self.create_violation(oToi, iSpaces)

    def number_of_spaces_is_gte(self):
        if self.number_of_spaces.startswith('>='):
            return True
        return False

    def analyze_gte_spaces(self, oToi):
        iSpaces = int(self.number_of_spaces[2:])
        iWhitespaces = extract_length_of_whitespace(oToi)
        if iWhitespaces < iSpaces:
            self.create_violation(oToi, iSpaces)

    def number_of_spaces_is_plus(self):
        if self.number_of_spaces.endswith('+'):
            return True
        return False

    def analyze_plus_spaces(self, oToi):
        iSpaces = int(self.number_of_spaces[0:-1])
        iWhitespaces = extract_length_of_whitespace(oToi)
        if iWhitespaces < iSpaces:
            self.create_violation(oToi, iSpaces)

    def number_of_spaces_is_an_integer(self):
        if isinstance(self.number_of_spaces, str):
            return False
        return True

    def analyze_integer_spaces(self, oToi):
        iWhitespaces = extract_length_of_whitespace(oToi)
        if iWhitespaces != self.number_of_spaces:
            self.create_violation(oToi, self.number_of_spaces)

    def create_violation(self, oToi, iNumSpaces):
        sSolution = create_solution(oToi, iNumSpaces)
        oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
        dAction = {}
        dAction['spaces'] = iNumSpaces
        oViolation.set_action(dAction)
        self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        if self.number_of_spaces == 0:
            lTokens = [lTokens[0], lTokens[2]]
        else:
            if isinstance(lTokens[1], parser.whitespace):
                lTokens[1].set_value(' '*dAction['spaces'])
            else:
                rules_utils.insert_whitespace(lTokens, dAction['spaces'])
        oViolation.set_tokens(lTokens)


def extract_length_of_whitespace(oToi):
    lTokens = oToi.get_tokens()
    iWhitespaces = len(lTokens[1].get_value())
    return iWhitespaces


def whitespace_token_exists(oToi):
    lTokens = oToi.get_tokens()
    if len(lTokens) == 2:
        return False
    if isinstance(lTokens[1], parser.whitespace):
        return True
    return False


def create_solution(oToi, iNumSpaces):
    lTokens = oToi.get_tokens()
    if whitespace_token_exists(oToi):
        sSolution = f'Change the number of spaces between {lTokens[0].get_value()} and {lTokens[2].get_value()} to {str(iNumSpaces)}'
    else:
        sSolution = f'Add {str(iNumSpaces)} spaces between {lTokens[0].get_value()} and {lTokens[1].get_value()}'
    return sSolution




from vsg import parser


class file_keyword(parser.keyword):
    '''
    unique_id = file_declaration : file_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class colon(parser.colon):
    '''
    unique_id = file_declaration : colon
    '''

    def __init__(self, sString=':'):
        parser.colon.__init__(self)


class semicolon(parser.semicolon):
    '''
    unique_id = file_declaration : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)


class keyword(parser.keyword):
    '''
    unique_id = file_declaration : keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class identifier(parser.identifier):
    '''
    unique_id = file_declaration : identifier
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)


class comma(parser.comma):
    '''
    unique_id = file_declaration : comma
    '''

    def __init__(self):
        parser.comma.__init__(self)


class subtype_indication(parser.subtype_indication):
    '''
    unique_id = file_declaration : subtype_indication
    '''

    def __init__(self, sString):
        parser.subtype_indication.__init__(self, sString)


class open_keyword(parser.keyword):
    '''
    unique_id = file_declaration : open_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class open_kind_expression(parser.expression):
    '''
    unique_id = file_declaration : open_kind_expression
    '''

    def __init__(self, sString):
        parser.expression.__init__(self, sString)


class is_keyword(parser.keyword):
    '''
    unique_id = file_declaration : is_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class logical_name(parser.logical_name):
    '''
    unique_id = file_declaration : logical_name
    '''

    def __init__(self, sString):
        parser.logical_name.__init__(self, sString)




from vsg import token

from vsg.rules import token_case as Rule

lTokens = []
lTokens.append(token.case_generate_statement.generate_keyword)


class rule_501(Rule):
    '''
    This rule checks the *generate* keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       case expression GENERATE

    **Fix**

    .. code-block:: vhdl

       case expression generate
    '''

    def __init__(self):
        Rule.__init__(self, 'case_generate_statement', '501', lTokens)
        self.groups.append('case::keyword')




from vsg.rules import remove_excessive_blank_lines_below_line_ending_with_token

from vsg import token

lTokens = []
lTokens.append(token.if_statement.else_keyword)

lOverrides = []
lOverrides.append(token.case_statement.case_label)
lOverrides.append(token.case_statement.case_keyword)
lOverrides.append(token.loop_statement.loop_label)
lOverrides.append(token.loop_statement.loop_keyword)
lOverrides.append(token.iteration_scheme.while_keyword)
lOverrides.append(token.iteration_scheme.for_keyword)


class rule_011(remove_excessive_blank_lines_below_line_ending_with_token):
    '''
    This rule checks for blank lines after the **else** keyword.

    **Violation**

    .. code-block:: vhdl

      else


        e <= '0';

    **Fix**

    .. code-block:: vhdl

      else
        e <= '0';
    '''
    def __init__(self):
        remove_excessive_blank_lines_below_line_ending_with_token.__init__(self, 'if', '011', lTokens, iAllow=0, lOverrides=lOverrides)
        self.solution = 'Remove blank line(s) after the *else* keyword.'




from vsg import parser
from vsg import token

from vsg.rules import consistent_token_case as Rule

lTokens = []
lTokens.append(token.alias_declaration.alias_designator)

lIgnore = []
lIgnore.append(token.interface_signal_declaration.identifier)
lIgnore.append(token.interface_unknown_declaration.identifier)
lIgnore.append(token.interface_constant_declaration.identifier)
lIgnore.append(token.interface_variable_declaration.identifier)
lIgnore.append(token.association_element.formal_part)
lIgnore.append(token.entity_declaration.identifier)
lIgnore.append(token.entity_declaration.entity_simple_name)
lIgnore.append(token.architecture_body.entity_name)
lIgnore.append(parser.whitespace)
lIgnore.append(parser.carriage_return)
lIgnore.append(parser.blank_line)


class rule_503(Rule):
    '''
    This rule checks for consistent capitalization of alias designators.

    **Violation**

    .. code-block:: vhdl

       architecture RTL of ENTITY1 is

         signal instructure : bit_vector(15 downto 0);
         alias opcode : bit_vector(3 downto 0) is instructure(15 downto 12);

         signal data : std_logic_vector(OPCODE'range);

       begin

         data <= OpCode;

         PROC_NAME : process () is
         begin

           data <= OpCOde;

           if (opCODE = "0110") then
             data <= oPCode;
           end if;

         end process PROC_NAME;

       end architecture RTL;

    **Fix**

    .. code-block:: vhdl

       architecture RTL of ENTITY1 is

         signal instructure : bit_vector(15 downto 0);
         alias opcode : bit_vector(3 downto 0) is instructure(15 downto 12);

         signal data : std_logic_vector(opcode'range);

       begin

         data <= opcode;

         PROC_NAME : process () is
         begin

           data <= opcode;

           if (opcode = "0110") then
             data <= opcode;
           end if;

         end process PROC_NAME;

       end architecture RTL;
    '''

    def __init__(self):
        Rule.__init__(self, 'alias_declaration', '503', lTokens, lIgnore)




from vsg import parser

from vsg.rules import remove_spaces_before_token_rule


class rule_003(remove_spaces_before_token_rule):
    '''
    This rule checks for spaces before semicolons.

    **Violation**

    .. code-block:: vhdl

       wr_en : in    std_logic      ;

    **Fix**

    .. code-block:: vhdl

       wr_en : in    std_logic;
    '''

    def __init__(self):
        remove_spaces_before_token_rule.__init__(self, 'whitespace', '003', parser.semicolon)
        self.solution = 'Remove spaces before semicolons.'




from vsg.rules.whitespace_between_tokens import Rule

from vsg.token import context_declaration as token


class rule_002(Rule):
    '''
    This rule checks for a single space between the **context** keyword and the context identifier.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       context   c1 is

    **Fix**

    .. code-block:: vhdl

       context c1 is
    '''
    def __init__(self):
        Rule.__init__(self, 'context', '002')
        self.left_token = token.context_keyword
        self.right_token = token.identifier




from vsg import parser
from vsg import violation

from vsg.rules import utils as rules_utils
from vsg.rule_group import structure
from vsg.vhdlFile import utils


class move_token_next_to_another_token_if_it_exists_between_tokens(structure.Rule):
    '''
    Moves one token next to another and places a single space between them.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    anchor_token : token type
       The token which another token will moved next to.

    token_to_move : token type
       The token which will be moved next to the anchor token.
    '''

    def __init__(self, name, identifier, anchor_token, token_to_move, between_tokens):
        structure.Rule.__init__(self, name, identifier)
        self.subphase = 2
        self.anchor_token = anchor_token
        self.token_to_move = token_to_move
        self.between_tokens = between_tokens

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_bounded_by(self.between_tokens[0], self.between_tokens[1], bIncludeTillEndOfLine=True)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            for iToken, oToken in enumerate(lTokens):
                if isinstance(oToken, self.anchor_token):
                    iStartIndex = iToken
                if isinstance(oToken, self.token_to_move):
                    iMoveIndex = iToken
                    if not (iStartIndex + 2 == iMoveIndex and isinstance(lTokens[iStartIndex + 1], parser.whitespace)):
                        oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
                        dAction = {}
                        dAction['insertIndex'] = iStartIndex + 1
                        dAction['moveIndex'] = iMoveIndex
                        oViolation.set_action(dAction)
                        oViolation.set_remap()
                        oViolation.fix_blank_lines = True
                        self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        oMoveToken = lTokens.pop(dAction['moveIndex'])
        rules_utils.insert_token(lTokens, dAction['insertIndex'], oMoveToken)
        rules_utils.insert_whitespace(lTokens, dAction['insertIndex'])
        lTokens = utils.fix_blank_lines(lTokens)
        oViolation.set_tokens(lTokens)




from vsg import parser


class units_keyword(parser.keyword):
    '''
    unique_id = physical_type_definition : units_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_keyword(parser.keyword):
    '''
    unique_id = physical_type_definition : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_units_keyword(parser.keyword):
    '''
    unique_id = physical_type_definition : end_units_definition
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class simple_name(parser.simple_name):
    '''
    unique_id = physical_type_definition : simple_name
    '''

    def __init__(self, sString):
        parser.simple_name.__init__(self, sString)




from vsg.vhdlFile.classify import range_constraint
from vsg.vhdlFile.classify import array_constraint
from vsg.vhdlFile.classify import record_constraint


def detect(iToken, lObjects):
    '''
    constraint ::=
        range_constraint
      | array_constraint
      | record_constraint
    '''

    iReturn = range_constraint.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = array_constraint.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = record_constraint.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    return iToken




import bisect
import pprint

from vsg import parser


class New():

    def __init__(self, dMap):
        self.dMap = dMap

    def get_token_indexes(self, oToken, bCopy=False):
        sBase, sSub = extract_unique_id(oToken)
        try:
            if bCopy:
                return self.dMap[sBase][sSub].copy()
            else:
                return self.dMap[sBase][sSub]
        except KeyError:
            return []

    def get_token_indexes_between_indexes(self, oToken, iStart, iEnd):
        lReturn = []
        lIndexes = self.get_token_indexes(oToken)
        for iIndex in lIndexes:
            if iIndex > iStart and iIndex < iEnd:
                lReturn.append(iIndex)
        return lReturn

    def get_line_number_of_index(self, iIndex):
        iLine = bisect.bisect_left(self.dMap['parser']['carriage_return'], iIndex) + 1
        return iLine

    def get_index_of_carriage_return_after_index(self, iIndex):
        iTemp = bisect.bisect_right(self.dMap['parser']['carriage_return'], iIndex)
        return self.dMap['parser']['carriage_return'][iTemp]

    def get_index_of_carriage_return_before_index(self, iIndex):
        if iIndex == 0:
            return None
        iTemp = bisect.bisect_left(self.dMap['parser']['carriage_return'], iIndex) - 1
        return self.dMap['parser']['carriage_return'][iTemp]

    def get_index_of_token_after_index(self, oToken, iIndex):
        sBase, sSub = extract_unique_id(oToken)
        try:
            iTemp = bisect.bisect_right(self.dMap[sBase][sSub], iIndex)
            return self.dMap[sBase][sSub][iTemp]
        except IndexError:
            return None
        except KeyError:
            return None

    def get_token_pair_indexes(self, oStart, oEnd):
        lStartIndexes = self.get_token_indexes(oStart)
        lEndIndexes = self.get_token_indexes(oEnd)
        return extract_start_end_indexes(lStartIndexes, lEndIndexes)

    def get_index_of_next_non_whitespace_token(self, iIndex, bExcludeComments=False):
        iStartIndex = iIndex + 1
        lTokens = [None, None, None, None]
        lBaseKeys = list(self.dMap.keys())
        for sBaseKey in lBaseKeys:
            lSubKeys = list(self.dMap[sBaseKey].keys())
            if sBaseKey == 'parser':
                if 'whitespace' in lSubKeys:
                    lSubKeys.remove('whitespace')
                if 'carriage_return' in lSubKeys:
                    lSubKeys.remove('carriage_return')
                if 'blank_line' in lSubKeys:
                    lSubKeys.remove('blank_line')
                if bExcludeComments and 'comment' in lSubKeys:
                    lSubKeys.remove('comment')
            for sSubKey in lSubKeys:
                for iIdx in range(0, 4):
                    iSearchIdx = iStartIndex + iIdx
                    if iSearchIdx in self.dMap[sBaseKey][sSubKey]:
                        lTokens[iIdx] = iSearchIdx
                        continue

        for iToken in lTokens:
            if iToken is not None:
                return iToken

    def get_index_of_previous_non_whitespace_token_before_index(self, iIndex):
        iStartIndex = iIndex - 1
        for i in range(iStartIndex, 0, -1):
            if not self.is_token_at_index_whitespace(i):
                return i

    def get_index_of_previous_non_whitespace_token(self, iIndex):
        iStartIndex = iIndex - 1
        for i in range(iStartIndex, 0, -1):
            if not self.is_token_at_index_whitespace_or_comment(i):
                return i

    def is_token_at_index(self, oToken, iIndex):
        try:
            sBase, sSub = extract_unique_id(oToken)
            if iIndex in self.dMap[sBase][sSub]:
                return True
            return False
        except KeyError:
            return False

    def is_token_at_index_whitespace(self, iIndex):
        if self.is_token_at_index(parser.whitespace, iIndex):
            return True
        if self.is_token_at_index(parser.carriage_return, iIndex):
            return True
        if self.is_token_at_index(parser.blank_line, iIndex):
            return True
        return False

    def is_token_at_index_whitespace_or_comment(self, iIndex):
        if self.is_token_at_index(parser.whitespace, iIndex):
            return True
        if self.is_token_at_index(parser.comment, iIndex):
            return True
        if self.is_token_at_index(parser.carriage_return, iIndex):
            return True
        if self.is_token_at_index(parser.blank_line, iIndex):
            return True
        return False

    def pretty_print(self):
        pp=pprint.PrettyPrinter(indent=4)
        pp.pprint(self.dMap)

    def get_index_of_line(self, iLine):
        return(self.dMap['parser']['carriage_return'][iLine - 2] + 1)


def extract_unique_id(oToken):
    lDoc = oToken.__doc__.split()
    for iDoc, sDoc in enumerate(lDoc):
        if sDoc == 'unique_id':
            return lDoc[iDoc + 2], lDoc[iDoc + 4]
    return None, None


def process_tokens(lTokens):
    dMap = build_default_map()
    for iToken, oToken in enumerate(lTokens):
        sBase, sSub = oToken.get_unique_id()
        if sBase is not None:
           try:
               dMap[sBase][sSub].append(iToken)
           except KeyError:
               try:
                   dMap[sBase][sSub] = []
                   dMap[sBase][sSub].append(iToken)
               except KeyError:
                   dMap[sBase] = {}
                   dMap[sBase][sSub] = []
                   dMap[sBase][sSub].append(iToken)
        if sBase == 'logical_operator':
            try:
                dMap[sBase][sBase].append(iToken)
            except KeyError:
                try:
                    dMap[sBase][sBase] = []
                    dMap[sBase][sBase].append(iToken)
                except KeyError:
                    dMap[sBase] = {}
                    dMap[sBase][sBase] = []
                    dMap[sBase][sBase].append(iToken)
            continue
        if sSub == 'comma':
            try:
                if iToken not in dMap['parser']['comma']:
                    dMap['parser']['comma'].append(iToken)
            except KeyError:
                try:
                    dMap['parser']['comma'] = []
                    dMap['parser']['comma'].append(iToken)
                except KeyError:
                    dMap['parser'] = {}
                    dMap['parser']['comma'] = []
                    dMap['parser']['comma'].append(iToken)
            continue
        if sSub == 'open_parenthesis':
            try:
                if iToken not in dMap['parser']['open_parenthesis']:
                    dMap['parser']['open_parenthesis'].append(iToken)
            except KeyError:
                try:
                    dMap['parser']['open_parenthesis'] = []
                    dMap['parser']['open_parenthesis'].append(iToken)
                except KeyError:
                    dMap['parser'] = {}
                    dMap['parser']['open_parenthesis'] = []
                    dMap['parser']['open_parenthesis'].append(iToken)
            continue

    return New(dMap)


def build_default_map():
    dMap = {}
    return dMap


def extract_start_end_indexes(lStartIndexes, lEndIndexes):
    mylStartIndexes = lStartIndexes.copy()
    mylEndIndexes = lEndIndexes.copy()
    lPairs = extract_pairs(mylStartIndexes, mylEndIndexes)

    return extract_indexes_from_pairs(lPairs)


def extract_pairs(lStartIndexes, lEndIndexes):
    lMyPairs = []
    while len(lEndIndexes) > 0 and len(lStartIndexes) > 0:
        iMin = lEndIndexes[-1] + 1
        lPair = []
        for iStart in lStartIndexes:
            lPair = extract_closest_pair(iStart, lEndIndexes, lPair, iMin)
        lMyPairs.append(lPair)

        if pair_is_empty(lPair):
            break

        lStartIndexes.remove(lPair[0])
        lEndIndexes.remove(lPair[1])

    return lMyPairs


def extract_closest_pair(iStart, lEndIndexes, lPair, iMin):
    for iEnd in lEndIndexes:
        if iStart > iEnd:
            continue
        if iEnd - iStart < iMin:
            lPair = [iStart, iEnd]
            iMin = iEnd - iStart
    return lPair


def extract_indexes_from_pairs(lPairs):
    lReturnStartIndexes = []
    lReturnEndIndexes = []
    for pair in lPairs:
        if pair_is_empty(pair):
            break
        lReturnStartIndexes.append(pair[0])
        lReturnEndIndexes.append(pair[1])

    lReturnStartIndexes.sort()
    lReturnEndIndexes.sort()

    return lReturnStartIndexes, lReturnEndIndexes


def pair_is_empty(lPair):
    if len(lPair) == 0:
        return True
    return False




from vsg.rules import token_prefix

from vsg import token

lTokens = []
lTokens.append(token.case_generate_statement.generate_label)
lTokens.append(token.case_generate_statement.end_generate_label)
lTokens.append(token.for_generate_statement.generate_label)
lTokens.append(token.for_generate_statement.end_generate_label)
lTokens.append(token.if_generate_statement.generate_label)
lTokens.append(token.if_generate_statement.end_generate_label)


class rule_017(token_prefix):
    '''
    This rule checks for valid prefixes on generate statement labels.
    The default prefix is *gen\_*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       label : case condition generate

    **Fix**

    .. code-block:: vhdl

       gen_label : case condition generate
    '''

    def __init__(self):
        token_prefix.__init__(self, 'generate', '017', lTokens)
        self.prefixes = ['gen_']




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment

from vsg.token import case_statement_alternative as token

lTokens = []
lTokens.append(token.assignment)


class rule_012(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment):
    '''
    This rule checks for code after the **=>** operator.

    **Violation**

    .. code-block:: vhdl

       when 0 => a <= '1';

    **Fix**

    .. code-block:: vhdl

       when 0 =>
         a <= '1';
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment.__init__(self, 'case', '012', lTokens)
        self.solution = 'Move code after the => to the next line.'




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.incomplete_type_declaration.identifier)
lTokens.append(token.full_type_declaration.identifier)


class rule_004(token_case_with_prefix_suffix):
    '''
    This rule checks the type identifier has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       type STATE_MACHINE is (idle, write, read, done);

    **Fix**

    .. code-block:: vhdl

       type state_machine is (idle, write, read, done);
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'type', '004', lTokens)
        self.configuration.append('prefix_exceptions')
        self.configuration.append('suffix_exceptions')
        self.groups.append('case::name')




from vsg import parser


class beginning(parser.comment):
    '''
    unique_id = delimited_comment : beginning
    '''

    def __init__(self, sString='/*'):
        parser.comment.__init__(self, sString)


class ending(parser.comment):
    '''
    unique_id = delimited_comment : ending
    '''

    def __init__(self, sString='*/'):
        parser.comment.__init__(self, sString)


class text(parser.item):
    '''
    unique_id = delimited_comment : text
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)




from vsg.token import attribute_specification as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import entity_specification
from vsg.vhdlFile.classify import expression


def detect(iToken, lObjects):
    '''
    attribute_specification ::=
        attribute attribute_designator of entity_specification is expression ;
    '''
    if utils.are_next_consecutive_tokens(['attribute', None, 'of'], iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('attribute', token.attribute_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token(token.attribute_designator, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('of', token.of_keyword, iCurrent, lObjects)

    iCurrent = entity_specification.classify(iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('is', token.is_keyword, iCurrent, lObjects)

    iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import align_tokens_in_region_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.file_declaration.colon)
lAlign.append(token.signal_declaration.colon)
lAlign.append(token.constant_declaration.colon)
lAlign.append(token.variable_declaration.colon)
lAlign.append(token.alias_declaration.colon)
lAlign.append(token.alias_declaration.is_keyword)

lUnless = []
lUnless.append([token.subprogram_body.is_keyword,token.subprogram_body.begin_keyword])


class rule_401(align_tokens_in_region_between_tokens_unless_between_tokens):
    '''
    This rule checks the colons are in the same column for all declarations in the block declarative part.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal sig1: natural;
       variable var2  : natural;
       constant c_period : time;
       file my_test_input : my_file_type;

    **Fix**

    .. code-block:: vhdl

       signal sig1        : natural;
       variable var2      : natural;
       constant c_period  : time;
       file my_test_input : my_file_type;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_unless_between_tokens.__init__(self, 'block', '401', lAlign, token.block_statement.block_keyword, token.block_statement.begin_keyword, lUnless)
        self.solution = 'Align colon.'
        self.subphase = 2




from vsg.token import association_element as token

from vsg.vhdlFile import utils


def detect(iCurrent, lObjects):
    '''
    association_element ::=
        [ formal_part => ] actual_part

    An association element will either end in a close parenthesis or a comma that is not within paranthesis.

    accociation_element [)|,]

    '''
    iOpenParenthesis = 0
    iCloseParenthesis = 0
    iToken = iCurrent
    while not utils.token_is_semicolon(iToken, lObjects):
        iToken = utils.find_next_token(iToken, lObjects)
        if utils.token_is_open_parenthesis(iToken, lObjects):
            iOpenParenthesis += 1
        if utils.token_is_close_parenthesis(iToken, lObjects):
            iCloseParenthesis += 1
        if iCloseParenthesis == iOpenParenthesis + 1:
            classify(iCurrent, iToken, lObjects, ')')
            return iToken
        if iCloseParenthesis == iOpenParenthesis:
            if utils.token_is_comma(iToken, lObjects):
                classify(iCurrent, iToken, lObjects, ',')
                return iToken
        iToken += 1
    return iToken


def classify(iStart, iEnd, lObjects, sEnd):
    iCurrent = iStart
    sPrint = ''
    for oObject in lObjects[iStart:iEnd + 1]:
        sPrint += oObject.get_value()
    # Classify formal part if it exists
    if utils.find_in_index_range('=>', iStart, iEnd, lObjects):
        iCurrent = utils.assign_tokens_until('=>', token.formal_part, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required('=>', token.assignment, iCurrent, lObjects)

    # Classify actual part
    for iCurrent in range(iCurrent, iEnd):
        if utils.is_item(lObjects, iCurrent):
            utils.assign_token(lObjects, iCurrent, token.actual_part)

    return iCurrent




from vsg import parser


class until_keyword(parser.keyword):
    '''
    unique_id = condition_clause : until_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)




from vsg import parser
from vsg import violation

from vsg.rule_group import blank_line
from vsg.vhdlFile import utils


class remove_excessive_blank_lines_above_line_starting_with_token(blank_line.Rule):
    '''
    Checks for excessive blank lines above a line starting with a given token

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : token object type list
       token object that a blank line below should appear

    iAllow : integer
       Maximum number of blank lines

    lOverrides : token object type list
       Ignore lines with given token types
    '''

    def __init__(self, name, identifier, lTokens, iAllow=1, lOverrides=None):
        blank_line.Rule.__init__(self, name=name, identifier=identifier)
        self.solution = 'Remove blank lines above'
        self.lTokens = lTokens
        self.iAllow = iAllow
        if lOverrides == None:
            self.lOverrides = []
        else:
            self.lOverrides = lOverrides

    def analyze(self, oFile):
        lToi = oFile.get_blank_lines_above_line_starting_with_token(self.lTokens)
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            iCount = 0
            iLine = 0
            for oToken in lTokens:
                if isinstance(oToken, parser.blank_line):
                    iCount += 1
                iLine = utils.increment_line_number(iLine, oToken)
            bOverride = check_if_override_exists(oFile, oToi.get_line_number() - iLine, self.lOverrides)
            if bOverride:
                iCount -= 1
            if iCount > self.iAllow:
                oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
                dAction = {}
                if bOverride:
                    dAction['index'] = 2*(self.iAllow + 1)
                else:
                    dAction['index'] = 2*self.iAllow
                oViolation.set_action(dAction)
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        lNewTokens = lTokens[0:dAction['index']]
        oViolation.set_tokens(lNewTokens)


def check_if_override_exists(oFile, iLine, lOverrides):

    oMyToi = oFile.get_line_preceeding_line(iLine)
    try:
        lTokens = oMyToi.get_tokens()

        for oOverride in lOverrides:
            if utils.does_token_type_exist_in_list_of_tokens(oOverride, lTokens):
                return True
        return False
    except AttributeError:
        return False




from vsg.rules import align_tokens_in_region_between_tokens as Rule

from vsg import token

lAlign = []
lAlign.append(token.case_generate_alternative.assignment)

oStart = token.case_generate_statement.case_keyword
oEnd = token.case_generate_statement.end_keyword


class rule_400(Rule):
    '''
    This rule checks the *=>* are aligned in case_generate_alternatives.

    .. NOTE:: The default configuration is :code:`compact_alignment`.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       gc : case I generate
         when 1 =>
         when n_order =>
         when others =>
       end generate gc;

    **Fix**

    .. code-block:: vhdl

       gc : case I generate
         when 1       =>
         when n_order =>
         when others  =>
       end generate gc;
    '''

    def __init__(self):
        Rule.__init__(self, 'case_generate_statement', '400', lAlign, oStart, oEnd)
        self.solution = 'Inconsistent alignment of "=>".'
        self.disable = True
        self.compact_alignment = True
        self.blank_line_ends_group = False
        self.comment_line_ends_group = False
        self.separate_generic_port_alignment = False
        self.configuration.remove('separate_generic_port_alignment')




from vsg.token import simple_variable_assignment as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import expression


def detect(iToken, lObjects):
    '''
    simple_variable_assignment ::=
        target := expression ;
    '''

    if utils.is_next_token_one_of(['when', 'if', 'elsif', 'else'], iToken, lObjects):
        return False
    if utils.find_in_range(':=', iToken, ';', lObjects):
        if utils.find_in_range('with', iToken, ';', lObjects):
            return False
        if utils.find_in_range('when', iToken, ';', lObjects):
            return False
        return True
    return False


def classify(iToken, lObjects):

    iCurrent = utils.assign_tokens_until(':=', token.target, iToken, lObjects)
    iCurrent = utils.assign_next_token_required(':=', token.assignment, iCurrent, lObjects)

    iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent



from vsg import parser


class range_keyword(parser.keyword):
    '''
    unique_id = index_subtype_definition : range_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class undefined_range(parser.undefined_range):
    '''
    unique_id = index_subtype_definition : undefined_range
    '''

    def __init__(self, sString='<>'):
        parser.undefined_range.__init__(self)




from vsg.rules.whitespace_after_token import Rule

from vsg.token import loop_statement as token

lTokens = []
lTokens.append(token.label_colon)


class rule_104(Rule):
    '''
    This rule checks if a label exists that a single space exists after the colon.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

         label :    for index in 4 to 23 loop
         label :  for index in 0 to 100 loop

    **Fix**

    .. code-block:: vhdl

         label : for index in 4 to 23 loop
         label : for index in 0 to 100 loop
    '''
    def __init__(self):
        Rule.__init__(self, 'loop_statement', '104', lTokens)




from vsg.rules import align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens

from vsg import parser
from vsg import token

lAlign = []
lAlign.append(parser.comment)

oStart = token.process_statement.process_keyword
oEnd = token.process_statement.begin_keyword

lSkip = []
lSkip.append(parser.comment)


class rule_034(align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens):
    '''
    This rule aligns inline comments between the end of the process sensitivity list and the process **begin** keyword.
    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       proc_1 : process () is

          variable counter : integer range 0 to 31;     -- Counts the number of frames received
          variable width   : natural range 0 to 255; -- Keeps track of the data word size

          variable size    : natural range 0 to 7; -- Keeps track of the frame size

       begin

    **Fix**

    .. code-block:: vhdl

       proc_1 : process () is

          variable counter : integer range 0 to 31;  -- Counts the number of frames received
          variable width   : natural range 0 to 255; -- Keeps track of the data word size

          variable size    : natural range 0 to 7;   -- Keeps track of the frame size

       begin
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens.__init__(self, 'process', '034', lAlign, oStart, oEnd, lSkip)
        self.solution = 'Align comment.'
        self.subphase = 3
        self.blank_line_ends_group = False
        self.comment_line_ends_group = False




from vsg.token import variable_declaration as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import identifier_list
from vsg.vhdlFile.classify import subtype_indication


def detect(iToken, lObjects):
    '''
    variable_declaration ::=
        [ shared ] variable identifier_list : subtype_indication [ := expression ] ;
    '''

    if utils.is_next_token('shared', iToken, lObjects):
        return classify(iToken, lObjects)
    elif utils.is_next_token('variable', iToken, lObjects):
        return classify(iToken, lObjects)

    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_if('shared', token.shared_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_required('variable', token.variable_keyword, iCurrent, lObjects)

    iCurrent = identifier_list.classify_until([':'], iCurrent, lObjects, token.identifier)

    iCurrent = utils.assign_next_token_required(':', token.colon, iCurrent, lObjects)

    iCurrent = subtype_indication.classify(iCurrent, lObjects)

    if utils.is_next_token(':=', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(':=', token.assignment_operator, iCurrent, lObjects)
        iCurrent = expression.classify_until([';'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)
    return iCurrent




from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_consecutive_lines_starting_with_token(search_token, min_num_lines, lAllTokens, oTokenMap):

    lReturn = []
    lSearchLines = get_line_numbers_of_tokens_which_start_line(search_token, oTokenMap)
    lGroups = group_lines(lSearchLines)
    lFilteredGroups = filter_groups_based_on_number_of_lines(lGroups, min_num_lines)

    for lGroup in lFilteredGroups:
        iStartLine = lGroup[0]
        iEndLine = lGroup[-1]
        iStartToken = oTokenMap.get_index_of_line(iStartLine)
        iEndToken = oTokenMap.get_index_of_carriage_return_after_index(oTokenMap.get_index_of_line(iEndLine))
        lTemp = lAllTokens[iStartToken:iEndToken]

        lReturn.append(tokens.New(iStartToken, iStartLine, lTemp))

    return lReturn


def get_line_numbers_of_tokens_which_start_line(search_token, oTokenMap):
    lSearchIndexes = utils.filter_tokens_which_start_a_line(search_token, oTokenMap)
    lSearchLines = []
    for iSearchIndex in lSearchIndexes:
        lSearchLines.append(oTokenMap.get_line_number_of_index(iSearchIndex))
    return lSearchLines


def group_lines(lLines):
    lReturn = []
    if len(lLines) == 0:
        return []
    lTemp = [lLines[0]]
    for iLine in lLines[1:]:
        if iLine == lTemp[-1] + 1:
            lTemp.append(iLine)
        else:
            lReturn.append(lTemp)
            lTemp = [iLine]

    lReturn.append(lTemp)

    return lReturn


def filter_groups_based_on_number_of_lines(lGroups, min_num_lines):
    lReturn = []
    for lGroup in lGroups:
        if len(lGroup) >= min_num_lines:
            lReturn.append(lGroup)
    return lReturn




from vsg.vhdlFile.classify import package_declarative_item


def detect(iToken, lObjects):
    '''
    package_declarative_part ::=
        { package_declarative_item }
    '''

    iLast = 0
    iCurrent = iToken
    while iLast != iCurrent:
        iLast = iCurrent
        iCurrent = package_declarative_item.detect(iCurrent, lObjects)
    return iCurrent




from vsg import parser


class relational_operator(parser.keyword):
    '''
    unique_id = relational_operator : relational_operator
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class equal(relational_operator):
    '''
    unique_id = relational_operator : equal
    '''

    def __init__(self, sString='='):
        relational_operator.__init__(self, '=')


class not_equal(relational_operator):
    '''
    unique_id = relational_operator : not_equal
    '''

    def __init__(self, sString='/='):
        relational_operator.__init__(self, '/=')


class less_than(relational_operator):
    '''
    unique_id = relational_operator : less_than
    '''

    def __init__(self, sString='<'):
        relational_operator.__init__(self, '<')


class less_than_or_equal(relational_operator):
    '''
    unique_id = relational_operator : less_than_or_equal
    '''

    def __init__(self, sString='<='):
        relational_operator.__init__(self, '<=')


class greater_than(relational_operator):
    '''
    unique_id = relational_operator : greater_than
    '''

    def __init__(self, sString='>'):
        relational_operator.__init__(self, '>')


class greater_than_or_equal(relational_operator):
    '''
    unique_id = relational_operator : greater_than_or_equal
    '''

    def __init__(self, sString='>='):
        relational_operator.__init__(self, '>=')


class question_equal(relational_operator):
    '''
    unique_id = relational_operator : question_equal
    '''

    def __init__(self, sString='?='):
        relational_operator.__init__(self, '?=')


class question_not_equal(relational_operator):
    '''
    unique_id = relational_operator : question_not_equal
    '''

    def __init__(self, sString='?/='):
        relational_operator.__init__(self, '?/=')


class question_less_than(relational_operator):
    '''
    unique_id = relational_operator : question_less_than
    '''

    def __init__(self, sString='?<'):
        relational_operator.__init__(self, '?<')


class question_less_than_or_equal(relational_operator):
    '''
    unique_id = relational_operator : question_less_than_or_equal
    '''

    def __init__(self, sString='?<='):
        relational_operator.__init__(self, '?<=')


class question_greater_than(relational_operator):
    '''
    unique_id = relational_operator : question_greater_than
    '''

    def __init__(self, sString='?>'):
        relational_operator.__init__(self, '?>')


class question_greater_than_or_equal(relational_operator):
    '''
    unique_id = relational_operator : question_greater_than_or_equal
    '''

    def __init__(self, sString='?>='):
        relational_operator.__init__(self, '?>=')






from vsg.rules import move_token_next_to_another_token

from vsg.token import architecture_body as token


class rule_005(move_token_next_to_another_token):
    '''
    This rule checks the **of** keyword is on the same line as the **architecture** keyword.

    **Violation**

    .. code-block:: vhdl

       architecture rtl
         of fifo is

    **Fix**

    .. code-block:: vhdl

       architecture rtl of
       fifo is
    '''

    def __init__(self):
        move_token_next_to_another_token.__init__(self, 'architecture', '005', token.identifier, token.of_keyword)
        self.solution = 'Ensure *of* keyword is on the same line as the architecture identifier.'




from vsg.vhdlFile.classify import subtype_indication


def classify(iToken, lObjects):
    '''
    element_subtype_definition ::=
        subtype_indication
    '''

    return subtype_indication.classify(iToken, lObjects)




from vsg.rules import blank_line_above_line_starting_with_token

from vsg import token

lTokens = []
lTokens.append(token.process_statement.end_keyword)


class rule_023(blank_line_above_line_starting_with_token):
    '''
    This rule checks for a blank line above the **end process** keyword.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

         wr_en <= '1';
       end process proc_a;

    **Fix**

    .. code-block:: vhdl

         wr_en <= '1';

       end process proc_a;
    '''

    def __init__(self):
        blank_line_above_line_starting_with_token.__init__(self, 'process', '023', lTokens)




from vsg import parser
from vsg import token

from vsg.rules import remove_lines_starting_with_token_between_token_pairs

lTokens = []
lTokens.append([token.simple_variable_assignment.target, token.simple_variable_assignment.semicolon])

oRemoveToken = parser.comment


class rule_006(remove_lines_starting_with_token_between_token_pairs):
    '''
    This rule checks for comments in multiline variable assignments.

    **Violation**

    .. code-block:: vhdl

         counter := 1 + 4 + 10 + 25 +
                    -- Add in more stuff
                    30 + 35;

    **Fix**

    .. code-block:: vhdl

         counter := 1 + 4 + 10 + 25 +
                    30 + 35;
    '''

    def __init__(self):
        remove_lines_starting_with_token_between_token_pairs.__init__(self, 'variable_assignment', '006', oRemoveToken, lTokens)
        self.solution = 'Remove comments inside variable assignment'
        self.fixable = False




from vsg import parser
from vsg import violation

from vsg.rule_group import whitespace as ws_group

from vsg.vhdlFile import utils


class rule_002(ws_group.Rule):
    '''
    This rule will check for the existence of tabs in the middle of a line.

    **Violation**

    .. code-block:: text

       \\t\\tsignal wr_en\\t:\\tstd_logic;  --\\tWrite Enable

    **Fix**

    .. code-block:: text

       \\t\\tsignal wr_en : std_logic;  -- Write Enable
    '''

    def __init__(self):

        ws_group.Rule.__init__(self, 'whitespace', '002')
        self.phase = 1
        self.solution = 'Remove tab'

    def _get_tokens_of_interest(self, oFile):
        lTemp = oFile.get_tokens_matching_not_at_beginning_or_ending_of_line([parser.whitespace])
        lToia = []
        for oToi in lTemp:
            lTokens = oToi.get_tokens()
            if lTokens[0].has_tab:
                lToia.append(oToi)

        lTemp = oFile.get_tokens_matching([parser.comment])
        lToib = []
        for oToi in lTemp:
            lTokens = oToi.get_tokens()
            if lTokens[0].has_tab:
                lToib.append(oToi)

        lReturn = utils.combine_two_token_class_lists(lToia, lToib)
        return lReturn

    def _analyze(self, lToi):
        for oToi in lToi:
            create_violation(oToi, self)

    def _fix_violation(self, oViolation):
        dAction = oViolation.get_action()
        if dAction['action'] == 'remove_tab_from_comment':
            remove_tab_from_comment(oViolation)
        else:
            remove_tab(oViolation)


def create_violation(oToi, self):
    lTokens = oToi.get_tokens()
    oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
    dAction = define_action(lTokens)
    oViolation.set_action(dAction)
    self.add_violation(oViolation)


def whitespace_exists(oToi):
    lTokens = oToi.get_tokens()
    if isinstance(lTokens[1], parser.whitespace):
        return True
    return False


def define_action(lTokens):
    dAction = {}
    if isinstance(lTokens[0], parser.comment):
        dAction['action'] = 'remove_tab_from_comment'
    else:
        dAction['action'] = 'remove_tab'
    return dAction


def need_to_remove_whitespace(dAction):
    if dAction['action'] == 'remove':
        return True
    return False


def remove_tab(oViolation):
    lTokens = oViolation.get_tokens()
    myToken = lTokens.pop()
    sValue = myToken.get_value()
    sValue = sValue.replace('\t', '  ')
    lTokens.append(parser.whitespace(sValue))
    lTokens[0].has_tab = False


def remove_tab_from_comment(oViolation):
    lTokens = oViolation.get_tokens()
    myToken = lTokens.pop()
    sValue = myToken.get_value()
    sValue = sValue.replace('\t', '  ')
    lTokens.append(parser.comment(sValue))
    lTokens[0].has_tab = False




from vsg.vhdlFile.classify import element_constraint


def detect(iToken, lObjects):
    '''
    array_element_constraint ::= element_constraint
    '''
    return element_constraint.detect(iToken, lObjects)




from vsg.rules import token_does_not_exist_before_token as Rule
from vsg.token import process_statement as token

oFirstToken = token.process_label

oSecondToken = token.process_keyword


class rule_016(Rule):
    '''
    This rule checks the process has a label.

    **Violation**

    .. code-block:: vhdl

       process (rd_en, wr_en, data_in, data_out,
                rd_full, wr_full
               ) is
       begin

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is
       begin
    '''

    def __init__(self):
        Rule.__init__(self, 'process', '016', oFirstToken, oSecondToken)
        self.solution = 'Add label for process statement'
        self.fixable = False




from vsg.rules import align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens

from vsg import parser
from vsg import token

lAlign = []
lAlign.append(parser.comment)

lSkip = []
lSkip.append(parser.comment)


class rule_412(align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens):
    '''
    This rule checks for alignment of inline comments for each parameter in the procedure declaration.

    |configuring_keyword_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

         procedure average_samples (
           constant a : in integer;   -- Comment about a
           signal d   : out std_logic;   -- Comment about d
         );

    **Fix**

    .. code-block:: vhdl

         procedure average_samples (
           constant a : in integer;    -- Comment about a
           signal d   : out std_logic; -- Comment about d
         );
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_skipping_lines_starting_with_tokens.__init__(
            self, 'procedure', '412', lAlign,
            token.procedure_specification.open_parenthesis, token.procedure_specification.close_parenthesis, lSkip)
        self.solution = 'Align comment.'
        self.subphase = 3




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.function_specification.pure_keyword)
lTokens.append(token.function_specification.impure_keyword)
lTokens.append(token.function_specification.function_keyword)


class rule_001(token_indent):
    '''
    This rule checks the indentation of the **function** keyword.

    **Violation**

    .. code-block:: vhdl

       architecture RTL of FIFO is

           function overflow (a: integer) return integer is


       function underflow (a: integer) return integer is

       begin

    **Fix**

    .. code-block:: vhdl

       architecture RTL of FIFO is

         function overflow (a: integer) return integer is

         function underflow (a: integer) return integer is

       begin
    '''

    def __init__(self):
        token_indent.__init__(self, 'function', '001', lTokens)




from vsg.vhdlFile.extract import tokens


def get_n_token_after_tokens(iToken, lTokens, lAllTokens, oTokenMap):
    lReturn = []
    lIndexes = []
    for oToken in lTokens:
        lTemp = oTokenMap.get_token_indexes(oToken)
        for iTemp in lTemp:
            iTokenIndex = iTemp
            for iCount in range(0, iToken):
                iTokenIndex = oTokenMap.get_index_of_next_non_whitespace_token(iTokenIndex, bExcludeComments=True)
            lIndexes.append(iTokenIndex)

    lIndexes.sort()

    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        lReturn.append(tokens.New(iIndex, iLine, [lAllTokens[iIndex]]))

    return lReturn




from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import concurrent_statement


def detect(iToken, lObjects):
    '''
    block_statement_part ::=
        { concurrent_statement }
    '''

    return utils.detect_submodule(iToken, lObjects, concurrent_statement)





from vsg.vhdlFile.classify import generic_clause
from vsg.vhdlFile.classify import port_clause


def detect(iToken, lObjects):
    '''
    entity_header ::=
        [ *formal*_generic_clause ]
        [ *formal*_port_clause ]
    '''

    iReturn = generic_clause.detect(iToken, lObjects)

    iReturn = port_clause.detect(iReturn, lObjects)

    return iReturn




from vsg import parser
from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([parser.close_parenthesis, token.if_statement.then_keyword])


class rule_004(Rule):
    '''
    This rule checks for a single space between the ) and the **then** keyword.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

      if (a = '1')then

      if (a = '1')    then

    **Fix**

    .. code-block:: vhdl

      if (a = '1') then

      if (a = '1') then
    '''
    def __init__(self):
        Rule.__init__(self, 'if', '004', lTokens)




from vsg.token import context_reference as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import utils as classify_utils


def detect(iCurrent, lObjects):
    '''
    context_reference ::=
        context selected_name { , selected_name } ;
    '''
    if utils.object_value_is(lObjects, iCurrent, 'context'):
        if not utils.find_in_range('is', iCurrent, ';', lObjects):
            return classify(iCurrent, lObjects)
    return iCurrent


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('context', token.keyword, iToken, lObjects)
    iCurrent = classify_utils.classify_selected_name(iCurrent, lObjects, token)
    while utils.is_next_token(',', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(',', token.comma, iCurrent, lObjects)
        iCurrent = classify_utils.classify_selected_name(iCurrent, lObjects, token)

    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)
    return iCurrent




from vsg.rules import move_token_next_to_another_token

from vsg.token import entity_declaration as token


class rule_005(move_token_next_to_another_token):
    '''
    This rule checks the **is** keyword is on the same line as the **entity** keyword.

    **Violation**

    .. code-block:: vhdl

       entity fifo

       entity fifo
         is

    **Fix**

    .. code-block:: vhdl

       entity fifo is

       entity fifo is
    '''

    def __init__(self):
        move_token_next_to_another_token.__init__(self, 'entity', '005', token.identifier, token.is_keyword)
        self.subphase = 2
        self.solution = 'Move *is* keyword next to identifier'




from vsg.rules import split_line_at_token_when_between_tokens

from vsg import token

lTokens = []
lTokens.append(token.generic_map_aspect.generic_keyword)

oStart = token.component_instantiation_statement.label_colon
oEnd = token.component_instantiation_statement.semicolon


class rule_012(split_line_at_token_when_between_tokens):
    '''
    This rule checks the instantiation declaration and the **generic map** keywords are not on the same line.

    **Violation**

    .. code-block:: vhdl

       U_FIFO : FIFO generic map (

    **Fix**

    .. code-block:: vhdl

       U_FIFO : FIFO
         generic map (
    '''

    def __init__(self):
        split_line_at_token_when_between_tokens.__init__(self, 'instantiation', '012', lTokens, oStart, oEnd)
        self.solution = 'Place *generic map* keywords on the next line by itself'




from vsg import parser
from vsg.token import process_statement as token

from vsg.rules import move_token_sequences_left_of_token

lSequences = []
lSequences.append([token.process_label, token.label_colon, token.process_keyword])
lSequences.append([token.process_label, token.label_colon, parser.whitespace, token.process_keyword])
lSequences.append([token.process_label, parser.whitespace, token.label_colon, token.process_keyword])
lSequences.append([token.process_label, parser.whitespace, token.label_colon, parser.whitespace, token.process_keyword])

oLeftToken = token.process_keyword


class rule_032(move_token_sequences_left_of_token):
    '''
    This rule checks the process label is on the same line as the process keyword.

    **Violation**

    .. code-block:: vhdl

       proc_1 :

       process(all) is

    **Fix**

    .. code-block:: vhdl

       proc_1 : process(all) is
    '''

    def __init__(self):
        move_token_sequences_left_of_token.__init__(self, 'process', '032', lSequences, oLeftToken)
        self.solution = 'Ensure process label is on the same line as *process* keyword.'




from vsg import parser
from vsg import token
from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import structure


class rule_030(structure.Rule):
    '''
    This rule checks for a single signal per line in a sensitivity list that is not the last one.
    The sensitivity list is required by the compiler, but provides no useful information to the reader.
    Therefore, the vertical spacing of the sensitivity list should be minimized.
    This will help with code readability.

    .. NOTE::  This rule is left to the user to fix.

    **Violation**

    .. code-block:: vhdl

       proc_a : process (rd_en,
                         wr_en,
                         data_in,
                         data_out,
                         rd_full,
                         wr_full
                        )

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        )
    '''

    def __init__(self):
        structure.Rule.__init__(self, 'process', '030')
        self.solution = 'Compact sensitivity list to reduce the number of lines it uses.'
        self.fixable = False

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_bounded_by(token.process_statement.open_parenthesis, token.process_statement.close_parenthesis)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            lTokens = utils.remove_whitespace_from_token_list(lTokens)[1:-1]
            lNewList = []
            iSignals = 0
            oSignal = None
            for oToken in lTokens:
                if isinstance(oToken, token.sensitivity_list.comma):
                    lNewList.append(parser.todo('signal name'))
                    iSignals += 1
                    oSignal = None
                elif isinstance(oToken, parser.carriage_return):
                    if oSignal is not None:
                        lNewList.append(parser.todo('signal name'))
                        iSignals += 1
                    lNewList.append(oToken)
                else:
                    oSignal = parser.todo('signal name')

            if oSignal is not None and not isinstance(oToken, parser.carriage_return):
                lNewList.append(parser.todo('signal name'))
                iSignals += 1


            if iSignals > 1:
                for iToken, oToken in enumerate(lNewList):
                    if iToken == 0:
                        continue
                    if isinstance(lNewList[iToken - 1], parser.todo) and isinstance(oToken, parser.todo):
                        break
                else:
                    oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
                    self.add_violation(oViolation)




from vsg.token import record_constraint as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import record_element_constraint


def detect(iToken, lObjects):
    '''
    record_constraint ::=
        ( record_element_constraint { , record_element_constraint } )
    '''
    if utils.is_next_token('(', iToken, lObjects):
        iTemp = utils.find_next_token(iToken, lObjects) + 1
        if record_element_constraint.detect(iTemp, lObjects):
            return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.assign_next_token_required('(', token.open_parenthesis, iToken, lObjects)

    while not utils.is_next_token(')', iCurrent, lObjects):
        iCurrent = record_element_constraint.classify(iCurrent, lObjects)
        iCurrent = utils.assign_next_token_if(',', token.comma, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(')', token.close_parenthesis, iCurrent, lObjects)

    return iCurrent




from vsg import token
from vsg import violation

from vsg.rule_group import structure


class formal_part_in_association_element_between_tokens(structure.Rule):
    '''
    Checks the case for words.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    oStart : token object type
       The beginning of the range

    oEnd : token object type
       The end of the range
    '''

    def __init__(self, name, identifier, oStart, oEnd):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.fixable = False
        self.oStart = oStart
        self.oEnd = oEnd

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_association_elements_between_tokens(self.oStart, self.oEnd)

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            bFound = False
            for oToken in lTokens:
                if isinstance(oToken, token.association_element.assignment):
                    bFound = True
                    break
            if not bFound:
                sSolution = self.solution
                oViolation = violation.New(oToi.get_line_number(), oToi, sSolution)
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        return None




from vsg.rules import blank_line_above_line_starting_with_token

from vsg.token import architecture_body as token


class rule_018(blank_line_above_line_starting_with_token):
    '''
    This rule checks for blank lines or comments above the **end architecture** declaration.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

         rd_en <= '1';
       end architecture RTL;

    **Fix**

    .. code-block:: vhdl

         rd_en <= '1';

       end architecture RTL;
    '''

    def __init__(self):
        blank_line_above_line_starting_with_token.__init__(self, 'architecture', '018', [token.end_keyword])




from vsg import deprecated_rule


class rule_001(deprecated_rule.Rule):
    '''
    This rule has been moved to **iteration_scheme_301**.
    '''

    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'for_loop', '001')
        self.message.append('Rule ' + self.unique_id + ' has been moved to iteration_scheme_301.')




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.context_declaration.context_simple_name)


class rule_016(token_case_with_prefix_suffix):
    '''
    This rule checks the context identifier has proper case in the end context declaration.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       end context C1;

    **Fix**

    .. code-block:: vhdl

       end context c1;
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'context', '016', lTokens)
        self.groups.append('case::name')




class ClassifyError(Exception):
    '''
    Exception raised for errors when classifying VHDL Files.
    '''

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message


class ConfigurationError(Exception):
    '''
    Exception raised for errors when configuration VSG..
    '''

    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message




from vsg.rules import token_case_with_prefix_suffix

from vsg.token import architecture_body as token

lTokens = []
lTokens.append(token.architecture_simple_name)


class rule_011(token_case_with_prefix_suffix):
    '''
    This rule checks the architecture name case in the **end architecture** statement.

    |configuring_uppercase_and_lowercase_rules_link|


    **Violation**

    .. code-block:: vhdl

       end architecture ARCHITECTURE_NAME;

    **Fix**

    .. code-block:: vhdl

       end architecture architecture_name;
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'architecture', '011', lTokens)
        self.groups.append('case::name')




from vsg import token

from vsg.rules.whitespace_before_token import Rule

lTokens = []
lTokens.append(token.variable_declaration.colon)


class rule_006(Rule):
    '''
    This rule checks for at least a single space before the colon.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       variable count: integer;
       variable counter : integer;

    **Fix**

    .. code-block:: vhdl

       variable count : integer;
       variable counter : integer;
    '''
    def __init__(self):
        Rule.__init__(self, 'variable', '006', lTokens)
        self.number_of_spaces = '>=1'




from vsg.rules import move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens

from vsg import token

lAnchorTokens = []
lAnchorTokens.append(token.context_declaration.end_keyword)
lAnchorTokens.append(token.context_declaration.end_context_keyword)
lAnchorTokens.append(token.context_declaration.context_simple_name)

oToken = token.context_declaration.semicolon

oStartToken = token.context_declaration.end_keyword
oEndToken = token.context_declaration.semicolon


class rule_011(move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens):
    '''
    This rule checks the semicolon is on the same line as the **end** keyword.

    **Violation**

    .. code-block:: vhdl

       end
       ;

       end context
       ;

       end context c1
       ;


    **Fix**

    .. code-block:: vhdl

       end;

       end context;

       end context c1;
    '''

    def __init__(self):
        move_token_to_the_right_of_several_possible_tokens_if_it_exists_between_tokens.__init__(self, 'context', '011', oToken, lAnchorTokens, oStartToken, oEndToken)
        self.subphase = 3




import string

from vsg import parser

from vsg.rule_group import structure

from vsg.vhdlFile import utils
from vsg.rules import utils as rules_utils


class Rule(structure.Rule):

    def __init__(self, name, identifier):
        structure.Rule.__init__(self, name=name, identifier=identifier)
        self.fixable = False
        self.disable = True

        self.min_height = 3
        self.configuration.append('min_height')
        self.allow_indenting = True
        self.configuration.append('allow_indenting')

    def _get_tokens_of_interest(self, oFile):
        lToi = oFile.get_consecutive_lines_starting_with_token(parser.comment, self.min_height)
        lReturn = []
        for oToi in lToi:
            iLeft, iLines, iRight = adjust_for_code_tags(oToi)
            if iLines >= self.min_height:
                lReturn.append(oToi.extract_tokens(iLeft, iRight))
        return lReturn

    def _analyze(self, lToi):

        for oToi in lToi:

            if not first_comment_is_a_header(oToi):
                continue

            self.analyze_comments(oToi)

    def fix(self, oFile, dFixOnly=None):
        '''
        Applies fixes for any rule violations.
        '''
        self.analyze(oFile)

    def set_token_indent(self, oToken):
        if self.allow_indenting:
            oToken.is_block_comment = False
        else:
            oToken.set_indent(0)
            oToken.is_block_comment = True
            oToken.block_comment_indent = 0

    def calculate_leading_whitespace(self, oToken):
        if self.allow_indenting:
            iWhitespace = self.indentSize * oToken.get_indent()
        else:
            iWhitespace = 0
        return iWhitespace

    def build_footer(self, oToken):
        iWhitespace = self.calculate_leading_whitespace(oToken)
        sFooter = '--'
        if self.footer_left is not None:
            sFooter += self.footer_left
            iFooter_left = len(self.footer_left)
        else:
            iFooter_left = 0

        if self.footer_string is None:
            sFooter += self.footer_left_repeat * (self.max_footer_column - iWhitespace - len(sFooter))
        elif self.footer_alignment == 'center':
            iLength = int((self.max_footer_column - iWhitespace - len(self.footer_string)) / 2) - iFooter_left - 2
            sFooter += self.footer_left_repeat * (iLength)
            sFooter += self.footer_string
            sFooter += self.footer_right_repeat * (self.max_footer_column - len(sFooter))
        elif self.footer_alignment == 'left':
            sFooter += self.footer_left_repeat
            sFooter += self.footer_string
            sFooter += self.footer_right_repeat * (self.max_footer_column - len(sFooter))
        elif self.footer_alignment == 'right':
            iLength = self.max_footer_column - iWhitespace - len(sFooter) - len(self.footer_string) - 1
            sFooter += self.footer_left_repeat * (iLength)
            sFooter += self.footer_string
            sFooter += self.footer_right_repeat
        return sFooter

    def build_comment(self, oToken):
        sHeader = '--'
        sHeader += self.comment_left
        return sHeader


def is_header(sComment):
    if bare_comment(sComment):
        return False
    if not third_character_is_alphanumeric(sComment):
        return False
    return fourth_character_is_alphanumeric(sComment)


def fourth_character_is_alphanumeric(sComment):
    try:
        if sComment[3] not in string.punctuation:
            return False
    except IndexError:
        return True
    return True


def third_character_is_alphanumeric(sComment):
    if sComment[2] not in string.punctuation:
        return False
    if sComment[2] == '!':
        return False
    return True


def bare_comment(sString):
    if sString == '--':
        return True
    return False


def is_footer(sComment):
    return is_header(sComment)


def first_comment_is_a_header(oToi):
    oToken = oToi.get_first_token_matching(parser.comment)
    if is_header(oToken.get_value()):
        return True
    return False


def adjust_for_code_tags(oToi):
    lTokens = oToi.get_tokens()
    iLeft, iLines, iRight = extract_initial_indexes_from_token_list(lTokens)
    iLeft, iLines = adjust_indexes_for_code_tags_at_beginning_of_block_comment(lTokens, iLeft, iLines)
    iRight, iLines = adjust_indexes_for_code_tags_at_end_of_block_comment(lTokens, iRight, iLines)
    return iLeft, iLines, iRight


def extract_initial_indexes_from_token_list(lTokens):
    iLines = utils.count_carriage_returns(lTokens) + 1
    iLeft = 0
    iRight = len(lTokens)
    return iLeft, iLines, iRight


def adjust_indexes_for_code_tags_at_beginning_of_block_comment(lTokens, iLeft, iLines):
    if code_tag_detected(lTokens[0]):
        iLeft = 2
        iLines -= 1
    if code_tag_detected(lTokens[1]):
        iLeft = 3
        iLines -= 1
    return iLeft, iLines


def adjust_indexes_for_code_tags_at_end_of_block_comment(lTokens, iRight, iLines):
    if code_tag_detected(lTokens[-1]):
        if rules_utils.token_is_whitespace(lTokens[-2]):
            iRight = len(lTokens) - 4
        else:
            iRight = len(lTokens) - 3
        iLines -= 1
    return iRight, iLines


def code_tag_detected(oToken):
    if 'vsg_on' in oToken.get_value() or 'vsg_off' in oToken.get_value():
        return True
    return False




from vsg.vhdlFile import utils

from vsg.token import case_generate_statement as token

from vsg.vhdlFile.classify import expression
from vsg.vhdlFile.classify import case_generate_alternative



def detect(iToken, lObjects):
    '''
        case_generate_statement ::=
            *generate*_label :
                case expression generate
                    case_generate_alternative
                    { case_generate_alternative }
                end generate [ *generate*_label ] ;
    '''

    if utils.are_next_consecutive_tokens([None, ':', 'case'], iToken, lObjects):
        return classify(iToken, lObjects)
    if utils.are_next_consecutive_tokens(['case'], iToken, lObjects):
        iIndex = utils.find_next_token(iToken, lObjects)
        oToken = token.case_keyword(lObjects[iToken].get_value())
        utils.print_error_message('generate_label', oToken, iIndex, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = utils.tokenize_label(iToken, lObjects, token.generate_label, token.label_colon)

    iCurrent = utils.assign_next_token_required('case', token.case_keyword, iCurrent, lObjects)

    iCurrent = expression.classify_until(['generate'], iCurrent, lObjects)

    iCurrent = utils.assign_next_token_required('generate', token.generate_keyword, iCurrent, lObjects)

    iToken = utils.detect_submodule(iToken, lObjects, case_generate_alternative)

    iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required('generate', token.end_generate_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if_not(';', token.end_generate_label, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import violation

from vsg.rule_group import structure


class Rule(structure.Rule):
    '''
    This rule checks if a token pair does not exist.
    '''

    def __init__(self, name, identifier, oFirstToken, oSecondToken):
        structure.Rule.__init__(self, name, identifier)
        self.oFirstToken = oFirstToken
        self.oSecondToken = oSecondToken

    def _get_tokens_of_interest(self, oFile):
        lReturn = []

        lFirstTokens = oFile.get_tokens_matching([self.oFirstToken])
        lSecondTokens = oFile.get_tokens_matching([self.oSecondToken])

        iPreviousIndex = 0
        for oSecondToken in lSecondTokens:
            iCurrentIndex = oSecondToken.get_start_index()
            if not first_token_before_second_token(lFirstTokens, iCurrentIndex, iPreviousIndex):
                lReturn.append(oSecondToken)
            iPreviousIndex = iCurrentIndex

        return lReturn

    def _analyze(self, lToi):
        for oToi in lToi:
            oViolation = violation.New(oToi.get_line_number(), oToi, self.solution)
            self.add_violation(oViolation)


def first_token_before_second_token(lFirstTokens, iCurrentIndex, iPreviousIndex):
    for oFirstToken in lFirstTokens:
        iFirstIndex = oFirstToken.get_start_index()
        if iPreviousIndex < iFirstIndex and iFirstIndex < iCurrentIndex:
            return True
    return False




from vsg import parser
from vsg import violation

from vsg.rules import utils
from vsg.rule_group import whitespace


class rule_100(whitespace.Rule):
    '''
    This rule checks for a single space after the **--**.

    **Violation**

    .. code-block:: vhdl

       --Comment 1
       --|Comment 2
       ---Comment
       ---------------------------

    **Fix**

    .. code-block:: vhdl

       -- Comment 1
       --| Comment 2
       --- Comment
       ---------------------------
    '''

    def __init__(self):
        whitespace.Rule.__init__(self, name='comment', identifier='100')
        self.solution = 'Undefined'
        self.phase = 2
        self.disable = False
        self.lTokens = [parser.comment]

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_matching(self.lTokens)

    def _analyze(self, lToi):
        for oToi in lToi:
            dResults = analyze_comment(oToi)
            if no_space_after_comment_keyword(dResults):
                create_violation(self, oToi, dResults)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()

        sToken = lTokens[0].get_value()
        sNewToken = sToken[0:dAction['index']] + ' ' + sToken[dAction['index']:]
        lTokens[0].set_value(sNewToken)
        oViolation.set_tokens(lTokens)


def no_space_after_comment_keyword(dResults):
    return dResults['violation']


def analyze_comment(oToi):
    sToken = get_comment_string(oToi)
    dAction = create_passing_action_dict()

    if not valid_comment(sToken):
        dAction = check_for_invalid_comment(dAction, sToken)

    return dAction


def create_passing_action_dict():
    dReturn = {}
    dReturn['violation'] = False
    return dReturn


def get_comment_string(oToi):
    lTokens = oToi.get_tokens()
    oToken = lTokens[0]
    return oToken.get_value()


def check_for_invalid_comment(dAction, sToken):
    if is_character_after_double_dash_a_letter_or_number(sToken):
        dAction = create_violation_action_dict(sToken, 2)
    elif is_a_header_candidate(sToken):
        dAction = create_violation_action_dict(sToken, 3)
    return dAction


def is_a_header_candidate(sToken):
    if len(sToken) < 4:
        return False
    if not sToken[3].isspace() and sToken[3].isalnum():
        return True
    return False


def is_character_after_double_dash_a_letter_or_number(sToken):
    if sToken[2].isalnum():
        return True
    return False


def create_violation_action_dict(sToken, iIndex):
    dReturn = {}
    dReturn['violation'] = True
    dReturn['index'] = iIndex
    dReturn['solution'] = create_solution(iIndex, sToken)
    return dReturn


def valid_comment(sToken):
    if len(sToken) == 2:
        return True
    if sToken.startswith('-- '):
        return True
    if sToken.startswith('---'):
        return True
    return False


def create_violation(self, oToi, dResults):
    oViolation = violation.New(oToi.get_line_number(), oToi, dResults['solution'])
    oViolation.set_action(dResults)
    self.add_violation(oViolation)


def create_solution(iIndex, sComment):
    return 'Change "' + sComment[0:iIndex + 1] + '" to "' + sComment[0:iIndex] + ' ' + sComment[iIndex] + '"'




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.use_clause.keyword)


class rule_005(token_case):
    '''
    This rule checks the **use** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       library ieee;
         USE ieee.std_logic_1164.all;
         Use ieee.std_logic_unsigned.all;

    **Fix**

    .. code-block:: vhdl

       library ieee;
         use ieee.std_logic_1164.all;
         use ieee.std_logic_unsigned.all;
    '''

    def __init__(self):
        token_case.__init__(self, 'library', '005', lTokens)
        self.groups.append('case::keyword')




from vsg import token

from vsg.rules.whitespace_after_token import Rule

lTokens = []
lTokens.append(token.entity_specification.colon)


class rule_100(Rule):
    '''
    This rule checks for a single space after the colon.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       attribute coordinate of comp_1 :component is (0.0, 17.5);
       attribute coordinate of comp_1 :    component is (0.0, 17.5);

    **Fix**

    .. code-block:: vhdl

       attribute coordinate of comp_1 : component is (0.0, 17.5);
       attribute coordinate of comp_1 : component is (0.0, 17.5);
    '''
    def __init__(self):
        Rule.__init__(self, 'entity_specification', '100', lTokens)




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.process_statement.begin_keyword)


class rule_010(split_line_at_token):
    '''
    This rule checks the **begin** keyword is on its own line.

    **Violation**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is begin

    **Fix**

    .. code-block:: vhdl

       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is
       begin
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'process', '010', lTokens)
        self.solution = 'Closing parenthesis must be on a line by itself.'




from vsg import token

from vsg.rules import move_token_left_to_next_non_whitespace_token as Rule

oToken = token.port_clause.semicolon


class rule_027(Rule):
    '''
    This rule checks the semicolon is not on its own line.

    **Violation**

    .. code-block:: vhdl

       U_FIFO : FIFO
         port (
           I_WIDTH : in integer
         )
         ;

    **Fix**

    .. code-block:: vhdl

       U_FIFO : FIFO
         generic (
           I_WIDTH : in integer
         );
    '''

    def __init__(self):
        Rule.__init__(self, 'port', '027', oToken)
        self.bInsertWhitespace = False




from vsg.vhdlFile.extract import tokens


def get_n_token_after_tokens_between_tokens(iToken, lTokens, oStart, oEnd, lAllTokens, oTokenMap):
    lReturn = []

    lStart, lEnd = oTokenMap.get_token_pair_indexes(oStart, oEnd)

    lIndexes = []
    for iStart, iEnd in zip(lStart, lEnd):
        for oToken in lTokens:
            lTemp = oTokenMap.get_token_indexes_between_indexes(oToken, iStart, iEnd)
            for iTemp in lTemp:
                iTokenIndex = iTemp
                for iCount in range(0, iToken):
                    iTokenIndex = oTokenMap.get_index_of_next_non_whitespace_token(iTokenIndex, bExcludeComments=True)
                lIndexes.append(iTokenIndex)

    lIndexes.sort()

    for iIndex in lIndexes:
        iLine = oTokenMap.get_line_number_of_index(iIndex)
        lReturn.append(tokens.New(iIndex, iLine, [lAllTokens[iIndex]]))

    return lReturn




from vsg import parser


class for_keyword(parser.keyword):
    '''
    unique_id = component_configuration : for_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class binding_indication_semicolon(parser.semicolon):
    '''
    unique_id = component_configuration : binding_indication_semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)


class end_keyword(parser.keyword):
    '''
    unique_id = component_configuration : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_for_keyword(parser.keyword):
    '''
    unique_id = component_configuration : end_for_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = component_configuration : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg import parser
from vsg import violation

from vsg.token import process_statement as token

from vsg.rule_group import blank_line

from vsg.rules import utils as rules_utils

from vsg.vhdlFile import utils


class rule_021(blank_line.Rule):
    '''
    This rule checks for blank lines above the **begin** keyword if there are no process declarative items.

    |configuring_blank_lines_link|

    **Violation**

    .. code-block:: vhdl

       proc_a : process

       begin


       proc_a : process (rd_en, wr_en)

       begin


       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is



       begin

    **Fix**

    .. code-block:: vhdl

       proc_a : process
       begin


       proc_a : process (rd_en, wr_en)
       begin


       proc_a : process (rd_en, wr_en, data_in, data_out,
                         rd_full, wr_full
                        ) is
       begin
    '''

    def __init__(self):
        blank_line.Rule.__init__(self, 'process', '021')
        self.solution = 'Remove blank lines above *begin* keyword'
        self.phase = 1
        self.style = 'no_blank_line_line'
        self.configuration.append('style')

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_tokens_bounded_by(token.process_keyword, token.begin_keyword)

    def _analyze(self, lToi):
        if self.style == 'no_blank_line':
            _analyze_no_blank_line(self, lToi)
        elif self.style == 'require_blank_line':
            _analyze_require_blank_line(self, lToi)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        if self.style == 'no_blank_line':
            lTokens.reverse()
            lNewTokens = []
            for iToken, oToken in enumerate(lTokens):
                if isinstance(oToken, parser.blank_line):
                    lNewTokens.pop()
                    continue
                lNewTokens.append(oToken)
            lNewTokens.reverse()
            oViolation.set_tokens(lNewTokens)
        elif self.style == 'require_blank_line':
            if isinstance(lTokens[-2], parser.whitespace):
                rules_utils.insert_blank_line(lTokens, -3)
                rules_utils.insert_carriage_return(lTokens, -3)
            else:
                rules_utils.insert_blank_line(lTokens, -2)
                rules_utils.insert_carriage_return(lTokens, -2)

            oViolation.set_tokens(lTokens)


def _analyze_no_blank_line(self, lToi):
    sSolution = 'Remove blank lines above *begin* keyword'
    dAction = {}
    dAction['action'] = 'Remove'
    for oToi in lToi:
        lTokens = oToi.get_tokens()
        iLine = oToi.get_line_number() + utils.count_carriage_returns(lTokens)
        for iToken, oToken in enumerate(lTokens):
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.process_keyword, token.begin_keyword], iToken, lTokens):
                if blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.close_parenthesis, token.begin_keyword], iToken, lTokens):
                if blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.is_keyword, token.begin_keyword], iToken, lTokens):
                if blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break

def _analyze_require_blank_line(self, lToi):
    sSolution = 'Add blank line above *begin* keyword'
    dAction = {}
    dAction['action'] = 'Insert'
    for oToi in lToi:
        lTokens = oToi.get_tokens()
        iLine = oToi.get_line_number() + utils.count_carriage_returns(lTokens)
        for iToken, oToken in enumerate(lTokens):
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.process_keyword, token.begin_keyword], iToken, lTokens):
                if not blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.close_parenthesis, token.begin_keyword], iToken, lTokens):
                if not blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break
            if utils.are_next_consecutive_token_types_ignoring_whitespace([token.is_keyword, token.begin_keyword], iToken, lTokens):
                if not blank_lines_exist(iToken, lTokens):
                    oViolation = violation.New(iLine, oToi, sSolution)
                    oViolation.set_action(dAction)
                    self.add_violation(oViolation)
                    break


def blank_lines_exist(iToken, lTokens):
    for iIndex in range(iToken, len(lTokens)):
        if isinstance(lTokens[iIndex], parser.blank_line):
            return True
    return False




from vsg.vhdlFile.extract import utils
from vsg.vhdlFile.extract import tokens


def get_line_which_includes_tokens(lTokens, lAllTokens, oTokenMap):

    lReturn = []

    lTokenIndexes = utils.get_indexes_of_token_list(lTokens, oTokenMap)

    for iIndex in lTokenIndexes:

        iStart = oTokenMap.get_index_of_carriage_return_before_index(iIndex)
        if iStart is None:
            iStart = 0
        else:
            iStart += 1

        iEnd = oTokenMap.get_index_of_carriage_return_after_index(iIndex)
        iLine = oTokenMap.get_line_number_of_index(iIndex)

        lTemp = lAllTokens[iStart:iEnd]

        oTokens = tokens.New(iStart, iLine, lTemp)
        oTokens.token_index = iIndex - iStart

        lReturn.append(oTokens)

    return lReturn




from vsg.token import entity_designator as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import signature


def classify(iToken, lObjects):
    '''
    entity_designator ::=
        entity_tag [ signature ]
    '''

    iCurrent = utils.assign_next_token(token.entity_tag, iToken, lObjects)

    iCurrent = signature.detect(iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_prefix

from vsg import token

lTokens = []
lTokens.append(token.incomplete_type_declaration.identifier)
lTokens.append(token.full_type_declaration.identifier)


class rule_015(token_prefix):
    '''
    This rule checks for valid prefixes in user defined type identifiers.
    The default new type prefix is *t\_*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       type my_type is range -5 to 5 ;

    **Fix**

    .. code-block:: vhdl

       type t_my_type is range -5 to 5 ;
    '''

    def __init__(self):
        token_prefix.__init__(self, 'type', '015', lTokens)
        self.prefixes = ['t_']
        self.solution = 'Type identifiers'




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.variable_declaration.shared_keyword)
lTokens.append(token.variable_declaration.variable_keyword)


class rule_001(token_indent):
    '''
    This rule checks the indent of variable declarations.

    **Violation**

    .. code-block:: vhdl

       proc : process () is

       variable count : integer;
             variable counter : integer;

       begin

    **Fix**

    .. code-block:: vhdl

       proc : process () is

         variable count : integer;
         variable counter : integer;

       begin
    '''

    def __init__(self):
        token_indent.__init__(self, 'variable', '001', lTokens)




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.entity_declaration.begin_keyword)


class rule_029(split_line_at_token):
    '''
    This rule checks the **begin** keyword is on its own line.

    **Violation**

    .. code-block:: vhdl

       entity FIFO is
         port (
           I_INPUT : in std_logic
         ); begin
       end entity;

    **Fix**

    .. code-block:: vhdl

       entity FIFO is
         port (
           I_INPUT : in std_logic
         );
       begin
       end entity;
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'entity', '029', lTokens)
        self.solution = 'Move the **begin** keyword to the next line.'




from vsg.rules import utils as rules_utils

from vsg.vhdlFile import utils


def fix_violation(oViolation):
    dAction = oViolation.get_action()
    if dAction['action'] == 'add_new_line':
        add_new_line(oViolation)
    elif dAction['action'] == 'remove_new_line':
        remove_new_line(oViolation)
    elif dAction['action'] == 'add_new_line_and_remove_carraige_returns':
        add_new_line_and_remove_carraige_returns(oViolation)


def add_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    rules_utils.remove_leading_whitespace_tokens(lTokens)
    rules_utils.insert_whitespace(lTokens, 0)
    rules_utils.insert_carriage_return(lTokens, 0)
    oViolation.set_tokens(lTokens)


def remove_new_line(oViolation):
    lTokens = oViolation.get_tokens()
    lTokens = utils.remove_carriage_returns_from_token_list(lTokens)
    utils.remove_consecutive_whitespace_tokens(lTokens)
    rules_utils.remove_leading_whitespace_tokens(lTokens)
    rules_utils.change_all_whitespace_to_single_character(lTokens)
    lNewTokens = utils.remove_trailing_whitespace(lTokens)
    utils.fix_blank_lines(lNewTokens)
    oViolation.set_tokens(lNewTokens)


def add_new_line_and_remove_carraige_returns(oViolation):
    lTokens = oViolation.get_tokens()
    lTokens = utils.remove_carriage_returns_from_token_list(lTokens)
    rules_utils.remove_leading_whitespace_tokens(lTokens)
    rules_utils.insert_whitespace(lTokens, 0)
    rules_utils.insert_carriage_return(lTokens, 0)
    rules_utils.change_all_whitespace_to_single_character(lTokens)
    utils.fix_blank_lines(lTokens)
    oViolation.set_tokens(lTokens)




from vsg import deprecated_rule


class rule_033(deprecated_rule.Rule):
    '''
    The function of this rule has been superceeced with comment indent updates and is handled by rule comment_010.
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'if', '033')
        self.message.append('The function of rule ' + self.unique_id + ' is covered by rule comment_010.')




from vsg import deprecated_rule


class rule_006(deprecated_rule.Rule):
    '''
    This rule has been deprecated and it's function was include in rule **port_005**.
    '''

    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'port', '006')
        self.message.append('Rule ' + self.unique_id + ' has been included in rule port_005.')




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.file_declaration.file_keyword)


class rule_002(token_case):
    '''
    This rule checks the **file** keyword has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       architecture rtl of fifo is

         FILE defaultImage : load_file_type open read_mode is load_file_name;

       begin

    **Fix**

    .. code-block:: vhdl

       architecture rtl of fifo is

         file defaultImage : load_file_type open read_mode is load_file_name;

       begin
    '''

    def __init__(self):
        token_case.__init__(self, 'file', '002', lTokens)
        self.groups.append('case::keyword')





from vsg import parser
from vsg import violation

from vsg.vhdlFile import utils
from vsg.rule_group import whitespace
from vsg.rules import utils as rules_utils


class n_spaces_before_and_after_tokens(whitespace.Rule):
    '''
    Checks for a single space between two tokens.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of token type pairs
       The tokens to check for a single space between
    '''

    def __init__(self, name, identifier, iSpaces, lTokens, bNIsMinimum=False):
        whitespace.Rule.__init__(self, name=name, identifier=identifier)
        self.iSpaces = iSpaces
        self.lTokens = lTokens
        self.bNIsMinimum = bNIsMinimum

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_n_tokens_before_and_after_tokens(2, self.lTokens)

    def _analyze(self, lToi):
        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)

            fStartLine = False
            if isinstance(lTokens[0], parser.carriage_return) and isinstance(lTokens[1], parser.whitespace):
                fStartLine = True

            myToi = oToi.extract_tokens(1, 3)

            iLine, lTokens = utils.get_toi_parameters(myToi)
            dAction = {}

            check_spaces_on_left_side(lTokens, fStartLine, self.bNIsMinimum, dAction, self.iSpaces)

            check_spaces_on_right_side(lTokens, self.bNIsMinimum, dAction, self.iSpaces)

            if len(list(dAction.keys())) > 0:
                sSolution = create_solution_text(dAction, self.iSpaces, lTokens)
                oViolation = violation.New(iLine, myToi, sSolution)
                oViolation.set_action(dAction)
                self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        lKeys = list(dAction.keys())
        for sKey in lKeys:
            if sKey == 'left':
                if dAction[sKey]['action'] == 'adjust':
                    lTokens[0].set_value(' '*self.iSpaces)
                else:
                    rules_utils.insert_whitespace(lTokens, 1)
            if sKey == 'right':
                if dAction[sKey]['action'] == 'adjust':
                    lTokens[-1].set_value(' '*self.iSpaces)
                else:
                    rules_utils.insert_whitespace(lTokens, len(lTokens) - 1)
        oViolation.set_tokens(lTokens)


def create_solution_text(dAction, iNumSpaces, lTokens):
    sReturn = ''
    for sKey in list(dAction.keys()):
        if sKey == 'left':
            if dAction[sKey]['action'] == 'adjust':
                sReturn += 'Remove all but one space before ' + lTokens[1].get_value() + '. '
            else:
                sReturn += 'Add ' + str(iNumSpaces) + ' space(s) before ' + lTokens[1].get_value() + '. '
        if sKey == 'right':
            if dAction[sKey]['action'] == 'adjust':
                sReturn += 'Remove all but one space after ' + lTokens[1].get_value()
            else:
                sReturn += 'Add ' + str(iNumSpaces) + ' space(s) after ' + lTokens[1].get_value()
    return sReturn


def check_spaces_on_left_side(lTokens, fStartLine, bNIsMinimum, dAction, iSpaces):
    if not fStartLine:
        oLeft = lTokens[0]
        if not isinstance(oLeft, parser.carriage_return):
            if isinstance(oLeft, parser.whitespace):
                if bNIsMinimum:
                    if iSpaces > len(oLeft.get_value()):
                        dAction['left'] = {}
                        dAction['left']['action'] = 'adjust'
                elif iSpaces != len(oLeft.get_value()):
                    dAction['left'] = {}
                    dAction['left']['action'] = 'adjust'
            else:
                dAction['left'] = {}
                dAction['left']['action'] = 'insert'


def check_spaces_on_right_side(lTokens, bNIsMinimum, dAction, iSpaces):
    oRight = lTokens[-1]
    if not isinstance(oRight, parser.carriage_return):
        if isinstance(oRight, parser.whitespace):
            if bNIsMinimum:
                if iSpaces > len(oRight.get_value()):
                    dAction['right'] = {}
                    dAction['right']['action'] = 'adjust'
            elif iSpaces != len(oRight.get_value()):
                dAction['right'] = {}
                dAction['right']['action'] = 'adjust'
        else:
            dAction['right'] = {}
            dAction['right']['action'] = 'insert'




from vsg import deprecated_rule


class rule_003(deprecated_rule.Rule):
    '''
    This rule was deprecated and replaced with rules:

    * `function_015 <function_rules.html#function-015>`_
    * `package_019 <package_rules.html#package-019>`_
    * `procedure_010 <procedure_rules.html#procedure-010>`_
    * `architecture_029 <architecture_rules.html#architecture-029>`_
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'signal', '003')
        self.message.append('Rule ' + self.unique_id + ' has been replaced with the following rules:')
        self.message.append('  function_015')
        self.message.append('  package_019')
        self.message.append('  procedure_010')
        self.message.append('  architecture_029')




from vsg import parser


class colon(parser.colon):
    '''
    unique_id = component_specification : colon
    '''

    def __init__(self, sString):
        parser.colon.__init__(self, sString)


class component_name(parser.name):
    '''
    unique_id = component_specification : component_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)




from vsg import severity


def print_output(dRunInfo):
    '''
    Displays results to stdout in syntastic format.

    Parameters:
      dRunInfo (dictionary)
    '''
    sOutputString = ''
    for dViolation in dRunInfo['violations']:
        sOutputString += _set_violation_type(dViolation)
        sOutputString += dRunInfo['filename']
        sOutputString += '('
        sOutputString += str(dViolation['lineNumber'])
        sOutputString += ')'
        sOutputString += dViolation['rule']
        sOutputString += ' -- '
        sOutputString += dViolation['solution']
        sOutputString += '\n'
    return sOutputString[:-1], None


def _set_violation_type(dViolation):
    if dViolation['severity']['type'] == severity.error_type:
        return 'ERROR: '
    elif dViolation['severity']['type'] == severity.warning_type:
        return 'WARNING: '




from vsg.rules import split_line_at_token

from vsg import token

lTokens = []
lTokens.append(token.block_statement.end_keyword)


class rule_006(split_line_at_token):
    '''
    This rule checks the **end** keyword is on its own line.

    **Violation**

    .. code-block:: vhdl

       a <= b; end block;

    **Fix**

    .. code-block:: vhdl

       a <= b;
       end block;
    '''

    def __init__(self):
        split_line_at_token.__init__(self, 'block', '006', lTokens)
        self.solution = 'Move *end* keyword and code after it to the next line'




from vsg.rules import token_case_in_range_bounded_by_tokens

from vsg import token

lTokens = []
lTokens.append(token.interface_constant_declaration.identifier)
lTokens.append(token.interface_signal_declaration.identifier)
lTokens.append(token.interface_variable_declaration.identifier)
lTokens.append(token.interface_unknown_declaration.identifier)


class rule_010(token_case_in_range_bounded_by_tokens):
    '''
    This rule checks the port names have proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       port (
         wr_en     : in    std_logic;
         rd_en     : in    std_logic;
         OVERFLOW  : out   std_logic;
         underflow : out   std_logic
       );

    **Fix**

    .. code-block:: vhdl

       port (
         wr_en     : in    std_logic;
         rd_en     : in    std_logic;
         overflow  : out   std_logic;
         underflow : out   std_logic
       );
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens.__init__(self, 'port', '010', lTokens, token.port_clause.open_parenthesis, token.port_clause.close_parenthesis)
        self.configuration.append('prefix_exceptions')
        self.configuration.append('suffix_exceptions')
        self.configuration.append('case_exceptions')
        self.groups.append('case::name')




from vsg import parser

from vsg.vhdlFile.extract import tokens


def get_tokens_bounded_by(oStart, oEnd, lAllObjects, oTokenMap, include_trailing_whitespace=False, bExcludeLastToken=False, bIncludeTillEndOfLine=False, bIncludeTillBeginningOfLine=False):

    lReturn = []
    lStart, lEnd = oTokenMap.get_token_pair_indexes(oStart, oEnd)

    lCarriageReturns = oTokenMap.get_token_indexes(parser.carriage_return)

    lNewStart = []
    if bIncludeTillBeginningOfLine:
        for iStart in lStart:
            lNewStart.append(oTokenMap.get_index_of_carriage_return_before_index(iStart) + 1)
    else:
        lNewStart = lStart

    lNewEnd = []
    if bIncludeTillEndOfLine:
        for iEnd in lEnd:
            lNewEnd.append(oTokenMap.get_index_of_carriage_return_after_index(iEnd))
    else:
        lNewEnd = lEnd

    if bExcludeLastToken:
        lNewEnd = [x - 1 for x in lNewEnd]

    lWhiteSpace = oTokenMap.get_token_indexes(parser.whitespace)
    if include_trailing_whitespace:
        for iNewEnd, iIndex in enumerate(lNewEnd):
            if iIndex + 1 in lWhiteSpace:
                lNewEnd[iNewEnd] +=1
    elif not bIncludeTillEndOfLine:
        for iNewEnd, iIndex in enumerate(lNewEnd):
            if iIndex in lWhiteSpace:
                lNewEnd[iNewEnd] -= 1
            if lNewEnd[iNewEnd] in lCarriageReturns:
                lNewEnd[iNewEnd] -= 1

    for iStart, iEnd, iIndex in zip(lNewStart, lNewEnd, lEnd):
        lTemp = lAllObjects[iStart: iEnd + 1]
        iStartLine = oTokenMap.get_line_number_of_index(iStart)
        oToi = tokens.New(iStart, iStartLine, lTemp)
        oToi.set_token_value(iIndex - iStart)
        lReturn.append(oToi)

        oStart = lAllObjects[iStart]
        oEnd = lAllObjects[iEnd]

    return lReturn




from vsg.rules import multiline_procedure_call_structure as Rule

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.procedure_call.procedure_name, token.concurrent_procedure_call_statement.semicolon])


class rule_003(Rule):
    '''
    This rule checks the structure of concurrent procedure calls.

    **Violation**

    .. code-block:: vhdl

       connect_ports(port_1 => data, port_2 => enable, port_3 => overflow, port_4 => underflow);

    **Fix**

    .. code-block:: vhdl

       connect_ports(
         port_1 => data,
         port_2 => enable,
         port_3 => overflow,
         port_4 => underflow
       );

    .. _procedure_call_whitespacing_rules:
    '''

    def __init__(self):
        Rule.__init__(self, 'procedure_call', '003')
        self.lTokenPairs = lTokenPairs




from vsg.rules import remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace

from vsg.token import concurrent_procedure_call_statement as token


class rule_002(remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace):
    '''
    This rule checks for labels on concurrent procedure call statements.
    Labels on procedure calls are optional and do not provide additional information.

    **Violation**

    .. code-block:: vhdl

       WR_EN_OUTPUT : WR_EN(parameter);

    **Fix**

    .. code-block:: vhdl

       WR_EN(parameter);
    '''

    def __init__(self):
        remove_tokens_bounded_by_tokens_and_remove_trailing_whitespace.__init__(self, 'procedure_call', '002', token.label_name, token.label_colon)
        self.solution = 'Remove Label'




from vsg import parser


class keyword(parser.keyword):
    '''
    unique_id = use_clause : keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class selected_name(parser.selected_name):
    '''
    unique_id = use_clause : selected_name
    '''

    def __init__(self, sString):
        parser.selected_name.__init__(self, sString)


class library_name(parser.name):
    '''
    unique_id = use_clause : library_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)


class package_name(parser.name):
    '''
    unique_id = use_clause : package_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)


class item_name(parser.name):
    '''
    unique_id = use_clause : item_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)


class comma(parser.comma):
    '''
    unique_id = use_clause : comma
    '''

    def __init__(self, sString=','):
        parser.comma.__init__(self)


class dot(parser.dot):
    '''
    unique_id = use_clause : dot
    '''

    def __init__(self, sString='.'):
        parser.dot.__init__(self)


class all_keyword(parser.keyword):
    '''
    unique_id = use_clause : all_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = use_clause : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg.vhdlFile.extract import tokens
from vsg.vhdlFile.extract import utils


def get_tokens_where_line_starts_with_token_until_ending_token_is_found(start_token, stop_token, lAllTokens, oTokenMap):

    lReturn = []

    lStartIndexes = utils.filter_tokens_which_start_a_line(start_token, oTokenMap)

    lStopIndexes = []
    for iStartIndex in lStartIndexes:
        lStopIndexes.append(oTokenMap.get_index_of_token_after_index(stop_token, iStartIndex))

    for iStart, iStop in zip(lStartIndexes, lStopIndexes):
        iLine = oTokenMap.get_line_number_of_index(iStart)
        lTemp = lAllTokens[iStart:iStop + 1]
        lReturn.append(tokens.New(iStart, iLine, lTemp))

    return lReturn




from vsg.rules import insert_token_right_of_token_if_it_does_not_exist_before_token

from vsg.token import context_declaration as token


class rule_021(insert_token_right_of_token_if_it_does_not_exist_before_token):
    '''
    This rule checks for the keyword **context** in the **end context** statement.

    |configuring_optional_items_link|

    **Violation**

    .. code-block:: vhdl

       end c1;

       end;

    **Fix**

    .. code-block:: vhdl

       end context c1;

       end context;
    '''

    def __init__(self):
        insert_token_right_of_token_if_it_does_not_exist_before_token.__init__(self, 'context', '021', token.end_context_keyword('context'), token.end_keyword, token.semicolon)
        self.solution = '*context* keyword'
        self.groups.append('structure::optional')




from vsg import token

from vsg.rules.whitespace_between_token_pairs import Rule

lTokens = []
lTokens.append([token.package_body.end_keyword, token.package_body.end_package_keyword])
lTokens.append([token.package_body.end_package_keyword, token.package_body.end_body_keyword])
lTokens.append([token.package_body.end_body_keyword, token.package_body.end_package_simple_name])


class rule_101(Rule):
    '''
    This rule checks for a single space between the **end**, **package** and **body** keywords and package name.

    |configuring_whitespace_rules_link|

    **Violation**

    .. code-block:: vhdl

       end   package   body    FIFO_PKG;

    **Fix**

    .. code-block:: vhdl

       end package  body   FIFO_PKG;
    '''
    def __init__(self):
        Rule.__init__(self, 'package_body', '101', lTokens)




from vsg.rules import token_case

from vsg import token

lTokens = []
lTokens.append(token.direction.to)


class rule_002(token_case):
    '''
    This rule checks the case of the **to** keyword.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       signal sig1 : std_logic_vector(3 TO 0);
       signal sig2 : std_logic_vector(16 tO 1);

    **Fix**

    .. code-block:: vhdl

       signal sig1 : std_logic_vector(3 to 0);
       signal sig2 : std_logic_vector(16 to 1);
    '''

    def __init__(self):
        token_case.__init__(self, 'range', '002', lTokens)
        self.groups.append('case::keyword')




from vsg import parser


class e_keyword(parser.keyword):
    '''
    unique_id = exponent : e_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class plus_sign(parser.item):
    '''
    unique_id = exponent : plus_sign
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)


class minus_sign(parser.item):
    '''
    unique_id = exponent : minus_sign
    '''

    def __init__(self, sString):
        parser.item.__init__(self, sString)


class integer(parser.integer):
    '''
    unique_id = exponent : integer
    '''

    def __init__(self, sString):
        parser.integer.__init__(self, sString)




from vsg.rules import token_indent_between_tokens

from vsg import token

lTokens = []
lTokens.append(token.use_clause.keyword)
lTokens.append(token.variable_declaration.variable_keyword)
lTokens.append(token.constant_declaration.constant_keyword)

oStart = token.subprogram_body.is_keyword
oEnd = token.subprogram_body.begin_keyword


class rule_005(token_indent_between_tokens):
    '''
    This rule checks the indent of lines between the **is** and **begin** keywords

    **Violation**

    .. code-block:: vhdl

       procedure average_samples (
         constant a : in integer;
         signal d : out std_logic ) is
       variable var_1 : integer;
           variable var_1 : integer;
       begin
       end procedure average_samples;


    **Fix**

    .. code-block:: vhdl

       procedure average_samples (
         constant a : in integer;
         signal b : in std_logic;
         variable c : in std_logic_vector(3 downto 0);
         signal d : out std_logic ) is
         variable var_1 : integer;
         variable var_1 : integer;
       begin
       end procedure average_samples;
    '''

    def __init__(self):
        token_indent_between_tokens.__init__(self, 'procedure', '005', lTokens, oStart, oEnd)




from vsg import deprecated_rule


class rule_021(deprecated_rule.Rule):
    '''
    This rule has been renamed to `port_map_009 <port_map_rules.html#port-map-009>`_.
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'instantiation', '021')
        self.message.append('Rule ' + self.unique_id + ' has been renamed to rule port_map_009.')




from vsg.rules import insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment

from vsg.token import port_clause as token

lTokens = []
lTokens.append(token.open_parenthesis)


class rule_016(insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment):
    '''
    This rule checks for a port definition on the same line as the **port** keyword.

    **Violation**

    .. code-block:: vhdl

       port (WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
       );

    **Fix**

    .. code-block:: vhdl

       port (
         WR_EN    : in    std_logic;
         RD_EN    : in    std_logic;
         OVERFLOW : out   std_logic;
         DATA     : inout std_logic
       );
    '''

    def __init__(self):
        insert_carriage_return_after_token_if_it_is_not_followed_by_a_comment.__init__(self, 'port', '016', lTokens)
        self.solution = 'Move port parameter to the next line.'



import argparse
import sys
from gettext import gettext as _


class CustomArgumentParser(argparse.ArgumentParser):
    """
    Custom class derived from argparse to modify the exit code when an error occurs.
    As told in https://www.python.org/dev/peps/pep-0389/#discussion-sys-stderr-and-sys-exit.
    """

    def error(self, message):
        self.print_usage(sys.stderr)
        args = {'prog': self.prog, 'message': message}
        self.exit(1, _('%(prog)s: error: %(message)s\n') % args)




from vsg import token

from vsg.rules import remove_carriage_return_after_token as Rule

lTokens = []
lTokens.append(token.loop_statement.label_colon)


class rule_005(Rule):
    '''
    This rule checks the loop label and the **while**, **for** or **loop** keywords are on the same line.

    **Violation**

    .. code-block:: vhdl

       LOOP_LABEL:
         loop

       LOOP_LABEL:
         while condition loop

       LOOP_LABEL:
         for x in range(15 downto 0) loop

    **Fix**

    .. code-block:: vhdl

       LOOP_LABEL: loop

       LOOP_LABEL: while condition loop

       LOOP_LABEL: for x in range(15 downto 0) loop
    '''

    def __init__(self):
        Rule.__init__(self, 'loop_statement', '005', lTokens, bInsertSpace=True)
        self.solution = 'Merge line below with label.'




from vsg import token

from vsg.rules import token_suffix

lTokens = []
lTokens.append(token.constant_declaration.identifier)


class rule_600(token_suffix):
    '''
    This rule checks for valid suffixes on constant identifiers.
    The default constant suffix is *\_c*.

    |configuring_prefix_and_suffix_rules_link|

    **Violation**

    .. code-block:: vhdl

       constant my_const : integer;

    **Fix**

    .. code-block:: vhdl

       constant my_const_c : integer;
    '''

    def __init__(self):
        token_suffix.__init__(self, 'constant', '600', lTokens)
        self.suffixes = ['_c']




from vsg.rules import blank_line_above_line_starting_with_token

from vsg import token

lTokens = []
lTokens.append(token.component_declaration.end_keyword)


class rule_016(blank_line_above_line_starting_with_token):
    '''
    This rule checks for blank lines above the **end component** line.

    **Violation**

    .. code-block:: vhdl

           overflow : std_logic
         );



       end component fifo;

    **Fix**

    .. code-block:: vhdl

           overflow : std_logic
         );
       end component fifo;
    '''

    def __init__(self):
        blank_line_above_line_starting_with_token.__init__(self, 'component', '016', lTokens)
        self.style = 'no_blank_line'




from vsg.rules import token_case_with_prefix_suffix

from vsg import token

lTokens = []
lTokens.append(token.attribute_specification.attribute_designator)


class rule_501(token_case_with_prefix_suffix):
    '''
    This rule checks the *attribute_designator* has proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       attribute COORDINATE of comp_1 : component is (0.0, 17.5);

    **Fix**

    .. code-block:: vhdl

       attribute coordinate of comp_1 : component is (0.0, 17.5);
    '''

    def __init__(self):
        token_case_with_prefix_suffix.__init__(self, 'attribute_specification', '501', lTokens)
        self.groups.append('case::name')




from vsg import deprecated_rule


class rule_025(deprecated_rule.Rule):
    '''
    This rule has been renamed to `port_map_003 <port_map_rules.html#port-map-003>`_.
    '''
    def __init__(self):
        deprecated_rule.Rule.__init__(self, 'instantiation', '025')
        self.message.append('Rule ' + self.unique_id + ' has been renamed to port_map_003.')




from vsg import parser

from vsg.token import waveform as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import waveform_element


def classify_until(lUntils, iToken, lObjects):
    '''
    waveform ::=
        waveform_element { , waveform_element }
      | unaffected
    '''

    if utils.is_next_token('unaffected', iToken, lObjects):
        return utils.assign_next_token_required('unaffected', token.unaffected_keyword, iToken, lObjects)

    iCurrent = iToken
    lMyUntils = lUntils
    lMyUntils.append(',')

    iCurrent = waveform_element.classify_until(lMyUntils, iCurrent, lObjects)

    while utils.is_next_token(',', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required(',', token.comma, iCurrent, lObjects)
        iCurrent = waveform_element.classify_until(lMyUntils, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_if(')', parser.todo, iCurrent, lObjects)

    return iCurrent




from vsg.rules import align_tokens_in_region_between_tokens_when_between_tokens_unless_between_tokens

from vsg import token

lAlign = []
lAlign.append(token.full_type_declaration.identifier)
lAlign.append(token.incomplete_type_declaration.identifier)
lAlign.append(token.file_declaration.identifier)
lAlign.append(token.constant_declaration.identifier)
lAlign.append(token.signal_declaration.identifier)
lAlign.append(token.subtype_declaration.identifier)
lAlign.append(token.variable_declaration.identifier)
lAlign.append(token.alias_declaration.alias_designator)

oStartToken = token.for_generate_statement.generate_keyword
oEndToken = token.generate_statement_body.begin_keyword

lBetweenTokens = []
lBetweenTokens.append([token.for_generate_statement.for_keyword, token.for_generate_statement.end_keyword])

lUnless = []
lUnless.append([token.subprogram_body.is_keyword,token.subprogram_body.begin_keyword])


class rule_400(align_tokens_in_region_between_tokens_when_between_tokens_unless_between_tokens):
    '''
    This rule checks the identifiers for all declarations are aligned in the generate declarative part in for generate statements.

    |configuring_identifier_alignment_rules_link|

    **Violation**

    .. code-block:: vhdl

       variable    var1 : natural;
       constant  c_period : time;

    **Fix**

    .. code-block:: vhdl

       variable var1     : natural;
       constant c_period : time;
    '''

    def __init__(self):
        align_tokens_in_region_between_tokens_when_between_tokens_unless_between_tokens.__init__(self, 'generate', '400', lAlign, oStartToken, oEndToken, lBetweenTokens, lUnless)
        self.solution = 'Align identifer.'




from vsg import parser


class configuration_keyword(parser.keyword):
    '''
    unique_id = configuration_body : configuration_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class identifier(parser.identifier):
    '''
    unique_id = configuration_body : identifier
    '''

    def __init__(self, sString):
        parser.identifier.__init__(self, sString)


class of_keyword(parser.keyword):
    '''
    unique_id = configuration_body : of_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class entity_name(parser.name):
    '''
    unique_id = configuration_body : entity_name
    '''

    def __init__(self, sString):
        parser.name.__init__(self, sString)


class is_keyword(parser.keyword):
    '''
    unique_id = configuration_body : is_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_keyword(parser.keyword):
    '''
    unique_id = configuration_body : end_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class end_configuration_keyword(parser.keyword):
    '''
    unique_id = configuration_body : end_configuration_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class configuration_simple_name(parser.simple_name):
    '''
    unique_id = configuration_body : configuration_simple_name
    '''

    def __init__(self, sString):
        parser.simple_name.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = configuration_body : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)





from vsg import parser
from vsg import violation

from vsg.rule_group import blank_line
from vsg.vhdlFile import utils


class remove_blank_lines_above_line_starting_with_token(blank_line.Rule):
    '''
    Checks for blank lines above a line.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : token list
       reference tokens to remove blank lines above
    '''

    def __init__(self, name, identifier, lTokens):
        blank_line.Rule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens

    def _get_tokens_of_interest(self, oFile):
        return oFile.get_consecutive_lines_starting_with_token_and_stopping_when_token_starting_line_is_found(parser.blank_line, self.lTokens[0])

    def _analyze(self, lToi):
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            iLine = oToi.get_line_number()

            for iToken, oToken in enumerate(lTokens):
               if isinstance(oToken, parser.carriage_return):
                   iLine += 1
                   for oSearchToken in self.lTokens:
                       if utils.are_next_consecutive_token_types([parser.whitespace, oSearchToken], iToken + 1, lTokens) or \
                          utils.are_next_consecutive_token_types([oSearchToken], iToken + 1, lTokens):
                           oViolation = violation.New(iLine, oToi, self.solution)
                           dAction = {}
                           dAction['remove_to_index'] = iToken + 1
                           oViolation.set_action(dAction)
                           self.add_violation(oViolation)

    def _fix_violation(self, oViolation):
        lTokens = oViolation.get_tokens()
        dAction = oViolation.get_action()
        lMyTokens = lTokens[dAction['remove_to_index']:]
        oViolation.set_tokens(lMyTokens)




from vsg.token import mode as token

from vsg.vhdlFile import utils


def classify(iToken, lObjects):
    '''
    mode ::=
        in | out | inout | buffer | linkage
    '''

    iCurrent = utils.assign_next_token_if('in', token.in_keyword, iToken, lObjects)
    iCurrent = utils.assign_next_token_if('out', token.out_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('inout', token.inout_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('buffer', token.buffer_keyword, iCurrent, lObjects)
    iCurrent = utils.assign_next_token_if('linkage', token.linkage_keyword, iCurrent, lObjects)

    return iCurrent




from vsg.rules import multiline_alignment_between_tokens

from vsg import token

lTokenPairs = []
lTokenPairs.append([token.concurrent_simple_signal_assignment.assignment, token.concurrent_simple_signal_assignment.semicolon])


class rule_003(multiline_alignment_between_tokens):
    '''
    This rule checks alignment of multiline concurrent simple signal assignments.
    Succesive lines should align to the space after the assignment operator.
    However, there is a special case if there are parenthesis in the assignment.
    If the parenthesis are not closed on the same line, then the next line will be aligned to the parenthesis.
    Aligning to the parenthesis improves readability.

    |configuring_multiline_indent_rules_link|

    **Violation**

    .. code-block:: vhdl

       O_FOO <= (1 => q_foo(63 downto 32),
                0 => q_foo(31 downto  0));

       n_foo <= resize(unsigned(I_FOO) +
                unsigned(I_BAR), q_foo'length);

    **Fix**

    .. code-block:: vhdl

       O_FOO <= (1 => q_foo(63 downto 32),
                 0 => q_foo(31 downto  0));

       n_foo <= resize(unsigned(I_FOO) +
                       unsigned(I_BAR), q_foo'length);
    '''

    def __init__(self):
        multiline_alignment_between_tokens.__init__(self, 'concurrent', '003', lTokenPairs)
        self.phase = 5
        self.subphase = 2



#!/usr/bin/env python

import argparse
import sys
import os

from . import config
from . import rule_list
from . import vhdlFile


def parse_command_line_arguments():
    '''Parses the command line arguments and returns them.'''

    parser = argparse.ArgumentParser(
      prog='VHDL Style Guide (VSG) Parser',
      description='''Outputs formatted rule documentation.''')

    return parser.parse_args()


def main():
    '''Main routine of parser output'''

    fExitStatus = 0

    commandLineArguments = parse_command_line_arguments()

    create_rule_documentation()

    sys.exit(fExitStatus)


def create_rule_documentation():

    oRuleList = build_rule_list()

    dRules = build_rule_dictionary(oRuleList)
    lRuleNames = get_names_of_rule_classes(oRuleList)
    for sRuleName in lRuleNames:
        build_rule_class_doc(sRuleName, dRules)

def build_rule_class_doc(sRuleName, dRules):
#    for sRuleName in lRuleName:
    lRuleClassDoc = []
    lRuleClassDoc.append('.. include:: includes.rst')
    lRuleClassDoc.extend(blank_line())
    if sRuleName == 'context_ref':
        sTitle = 'Context Reference Rules'
    elif sRuleName == 'exit_statement':
        sTitle = 'Exit Rules'
    else:
        sTitle = (sRuleName.title() + ' Rules').replace('_', ' ')
    lRuleClassDoc.append(sTitle)
    lRuleClassDoc.append('-'*len(sTitle))
    lRuleClassDoc.extend(blank_line())
    lRuleClassDoc.extend(import_preamble_doc(sRuleName))
    lRuleClassDoc.extend(do_something(list(dRules[sRuleName])))

    write_file(f'{sRuleName}_rules.rst', lRuleClassDoc)


def import_preamble_doc(sRuleName):
    lReturn = []
    if sRuleName == 'range':
        sFileName = f'vsg/rules/ranges/preamble_doc.rst'
    else:
        sFileName = f'vsg/rules/{sRuleName}/preamble_doc.rst'
    if os.path.exists(sFileName):
        lReturn = read_file(sFileName)
        lReturn.extend(blank_line())
    return lReturn


def build_rule_list():
    oVhdlFile = vhdlFile.vhdlFile([''])
    oConfig = config.config()
    return rule_list.rule_list(oVhdlFile, oConfig)


def build_rule_dictionary(oRuleList):
    lRuleName = []
    dRules = {}
    for oRule in oRuleList.rules:
        if oRule.name not in lRuleName:
            lRuleName.append(oRule.name)
            dRules[oRule.name] = []
        dRules[oRule.name].append(oRule)

    return dRules


def get_names_of_rule_classes(oRuleList):
    lRuleName = []
    for oRule in oRuleList.rules:
        if oRule.name not in lRuleName:
            lRuleName.append(oRule.name)
    return lRuleName


def do_something(lRules):
    lRuleDoc = []
    for oRule in lRules:
        lRuleDoc.extend(generate_rule_header(oRule))
        lRuleDoc.extend(blank_line())
        if not oRule.deprecated and not oRule.proposed:
            lRuleDoc.extend(generate_icons(oRule))
            lRuleDoc.extend(blank_line())
        lRuleDoc.extend(add_doc_string(oRule))
    return lRuleDoc


def add_doc_string(oRule):
    lReturn = []
    sReturn = oRule.__doc__
    iFirstCharacter = find_index_of_first_character(sReturn)
    sReturn = sReturn[iFirstCharacter:]
    sReturn = sReturn.replace('\n' + ' '*(iFirstCharacter-1), '\n')
    sReturn = sReturn.replace('\n' + ' '*(iFirstCharacter-2) + '\n', '\n\n')
    sReturn = sReturn.replace('[Violation]', '**Violation**\n\n.. code-block:: vhdl')
    sReturn = sReturn.replace('[Fix]', '**Fix**\n\n.. code-block:: vhdl')
    lReturn.append(sReturn)
    return lReturn

def find_index_of_first_character(sReturn):
    for iChar, sChar in enumerate(sReturn):
        if sChar != ' ' and sChar != '\n':
            return iChar
    return None


def generate_rule_header(oRule):
    lReturn = []
    lReturn.append(oRule.unique_id)
    lReturn.append('#'*len(oRule.unique_id))
    return lReturn


def blank_line():
    lReturn = []
    lReturn.append('')
    return lReturn


def write_file(sFilename, lLines):
    with open(sFilename, 'w') as oFile:
        for sLine in lLines:
            oFile.write(sLine + '\n')


def read_file(sFileName):
    lLines = []
    with open(sFileName) as oFile:
        for sLine in oFile:
            lLines.append(sLine.rstrip())
    return lLines


def generate_icons(oRule):
    sIcons = ''
    sIcons += create_phase_icon(oRule)
    sIcons += create_disabled_icon(oRule)
    sIcons += create_severity_icon(oRule)
    sIcons += create_group_icons(oRule)
    return [sIcons]


def create_phase_icon(oRule):
    return '|phase_' + str(oRule.phase) + '|'


def create_disabled_icon(oRule):
    sReturn = ''
    if oRule.disable:
        sReturn += (' ')
        sReturn += ('|disabled|')
    return sReturn


def create_severity_icon(oRule):
    sReturn = ' '
    sReturn += ('|' + oRule.severity.name.lower() + '|')
    return sReturn


def create_group_icons(oRule):
    sReturn = ''
    for sGroup in oRule.groups:
        sReturn += ' '
        sReturn += '|' + sGroup.replace('::', '_') + '|'
    return sReturn


if __name__ == '__rule_doc_gen__':
    main()




from vsg import parser


class with_keyword(parser.keyword):
    '''
    unique_id = selected_variable_assignment : with_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class select_keyword(parser.keyword):
    '''
    unique_id = selected_variable_assignment : select_keyword
    '''

    def __init__(self, sString):
        parser.keyword.__init__(self, sString)


class question_mark(parser.question_mark):
    '''
    unique_id = selected_variable_assignment : question_mark
    '''

    def __init__(self, sString='?'):
        parser.question_mark.__init__(self)


class target(parser.target):
    '''
    unique_id = selected_variable_assignment : target
    '''

    def __init__(self, sString):
        parser.target.__init__(self, sString)


class assignment(parser.assignment):
    '''
    unique_id = selected_variable_assignment : assignment
    '''

    def __init__(self, sString):
        parser.assignment.__init__(self, sString)


class semicolon(parser.semicolon):
    '''
    unique_id = selected_variable_assignment : semicolon
    '''

    def __init__(self, sString=';'):
        parser.semicolon.__init__(self)




from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import simple_force_assignment
from vsg.vhdlFile.classify import simple_release_assignment
from vsg.vhdlFile.classify import simple_waveform_assignment


def detect(iToken, lObjects):
    '''
    simple_signal_assignment ::=
        simple_waveform_assignment
      | simple_force_assignment
      | simple_release_assignment
    '''

    if utils.find_in_next_n_tokens('if', 3, iToken, lObjects):
        return False
    if utils.find_in_range('<=', iToken, ';', lObjects):
        if utils.find_in_range('when', iToken, ';', lObjects):
            return False
        if utils.find_in_range('with', iToken, ';', lObjects):
            return False
        return True
    return False


def classify(iToken, lObjects):

    iCurrent = iToken
    iCurrent = simple_force_assignment.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = simple_release_assignment.detect(iToken, lObjects)
    if iCurrent != iToken:
        return iCurrent

    iCurrent = simple_waveform_assignment.detect(iToken, lObjects)

    return iCurrent




from vsg.rules import token_case_in_range_bounded_by_tokens

from vsg import token

lTokens = []
lTokens.append(token.port_map_aspect.port_keyword)
lTokens.append(token.port_map_aspect.map_keyword)

oStart = token.component_instantiation_statement.label_colon
oEnd = token.component_instantiation_statement.semicolon


class rule_001(token_case_in_range_bounded_by_tokens):
    '''
    This rule checks the **port map** keywords have proper case.

    |configuring_uppercase_and_lowercase_rules_link|

    **Violation**

    .. code-block:: vhdl

       PORT MAP (

    **Fix**

    .. code-block:: vhdl

       port map (
    '''

    def __init__(self):
        token_case_in_range_bounded_by_tokens.__init__(self, 'port_map', '001', lTokens, oStart, oEnd)
        self.groups.append('case::keyword')




from vsg.vhdlFile.classify import attribute_specification
from vsg.vhdlFile.classify import group_declaration
from vsg.vhdlFile.classify import use_clause


def detect(iToken, lObjects):
    '''
    configuration_declarative_item ::=
        use_clause
      | attribute_specification
      | group_declaration
    '''

    iReturn = use_clause.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = attribute_specification.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    iReturn = group_declaration.detect(iToken, lObjects)
    if iReturn != iToken:
        return iReturn

    return iToken




from vsg.vhdlFile import utils

from vsg.token import simple_configuration_specification as token

from vsg.vhdlFile.classify import component_specification
from vsg.vhdlFile.classify import binding_indication


def detect(iToken, lObjects):
    '''
    simple_configuration_specification ::=
        **for** component_specification binding_indication ;
        [ **end** **for** ; ]
    '''
    if utils.is_next_token('for', iToken, lObjects):
        return classify(iToken, lObjects)
    return iToken


def classify(iToken, lObjects):

    iCurrent = iToken

    iCurrent = utils.assign_next_token_required('for', token.for_keyword, iCurrent, lObjects)

    iCurrent = component_specification.classify(iCurrent, lObjects)
    iCurrent = binding_indication.classify(iCurrent, lObjects)
    iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    if utils.is_next_token('end', iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required('for', token.end_for_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg import parser
from vsg import violation

from vsg.rules.whitespace_between_tokens import Rule as WhitespaceRule
from vsg.rules import utils as rules_utils


class Rule(WhitespaceRule):
    '''
    Checks for at least a single space before a token.

    Parameters
    ----------

    name : string
       The group the rule belongs to.

    identifier : string
       unique identifier.  Usually in the form of 00N.

    lTokens : list of token object types
       reference token to check for a whitespace before

    oStart : token object type
       The beginning of the range

    oEnd : token object type
       The end of the range
    '''

    def __init__(self, name, identifier, lTokens, oStart, oEnd):
        WhitespaceRule.__init__(self, name=name, identifier=identifier)
        self.lTokens = lTokens
        self.oStart = oStart
        self.oEnd = oEnd

    def _get_tokens_of_interest(self, oFile):
        lReturn = []
        lToi = oFile.get_token_and_n_tokens_before_it_in_between_tokens(self.lTokens, 2, self.oStart, self.oEnd)
        for oToi in lToi:
            lTokens = oToi.get_tokens()
            if rules_utils.token_is_at_beginning_of_line(lTokens):
                continue
            lReturn.append(extract_toi(oToi))
        return lReturn


def extract_toi(oToi):
    lTokens = oToi.get_tokens()
    if isinstance(lTokens[1], parser.whitespace):
        return oToi
    else:
        return oToi.extract_tokens(1, 2)




from vsg.rules import multiline_alignment_between_tokens as Rule

from vsg import parser
from vsg import token
from vsg.vhdlFile import utils
from vsg.rules import alignment_utils
from vsg.rules import utils as rules_utils

lTokenPairs = []
lTokenPairs.append([token.constant_declaration.assignment_operator, token.constant_declaration.semicolon])


class rule_014(Rule):
    '''
    This rule checks the indent of multiline constants that do not contain arrays.

    |configuring_multiline_indent_rules_link|

    **Violation**

    .. code-block:: vhdl

       constant width : integer := a + b +
         c + d;

    **Fix**

    .. code-block:: vhdl

       constant width : integer := a + b +
                                   c + d;
    '''

    def __init__(self):
        Rule.__init__(self, 'constant', '014', lTokenPairs)
        self.subphase = 3
        self.phase = 5
        self.iIndentAfterParen = 0

    def _get_tokens_of_interest(self, oFile):
        for lTokenPair in self.lTokenPairs:
            lToi = oFile.get_tokens_bounded_by(lTokenPair[0], lTokenPair[1])
            lToi = remove_arrays(lToi)

        for oToi in lToi:
            iLine, lTokens = utils.get_toi_parameters(oToi)
            iFirstLine, iFirstLineIndent = alignment_utils.get_first_line_info(iLine, oFile)
            iAssignColumn = oFile.get_column_of_token_index(oToi.get_start_index())
            oToi.set_meta_data('iIndent', _get_indent_of_line(iLine, oFile))
            oToi.set_meta_data('iFirstLine', iFirstLine)
            oToi.set_meta_data('iFirstLineIndent', iFirstLineIndent)
            oToi.set_meta_data('iAssignColumn', iAssignColumn)
            oToi.set_meta_data('indentSize', self.indentSize)
            oToi.set_meta_data('bStartsWithParen', alignment_utils.starts_with_paren(lTokens))

        return lToi


def remove_arrays(lToi):
    lReturn = []
    for oToi in lToi:
        lTokens = oToi.get_tokens()
        if not rules_utils.token_list_starts_with_paren(lTokens, 1):
            lReturn.append(oToi)
    return lReturn


def _get_indent_of_line(iLine, oFile):
    lTemp = oFile.get_tokens_from_line(iLine)
    oToken = lTemp.get_tokens()[0]

    if isinstance(oToken, parser.whitespace):
        oToken = lTemp.get_tokens()[1]
        return oToken.indent
    else:
        oToken = lTemp.get_tokens()[0]
        return oToken.indent




from vsg.token import generate_statement_body as token

from vsg.vhdlFile import utils

from vsg.vhdlFile.classify import block_declarative_part
from vsg.vhdlFile.classify import concurrent_statement


def classify(iToken, lObjects):
    '''
    generate_statement_body ::=
            [ block_declarative_part
        begin ]
            { concurrent_statement }
        [ end [ alternative_label ] ; ]
    '''
    iCurrent = utils.find_next_token(iToken, lObjects)
    iLast = iCurrent
    iCurrent = block_declarative_part.detect(iCurrent, lObjects)

    if iCurrent != iLast:
        iCurrent = utils.assign_next_token_required('begin', token.begin_keyword, iCurrent, lObjects)

    iCurrent = utils.assign_next_token_if('begin', token.begin_keyword, iCurrent, lObjects)

    iLast = 0
    while iCurrent != iLast:
        iLast = iCurrent
        if utils.is_next_token_one_of(['elsif', 'else', 'when'], iCurrent, lObjects):
            return iCurrent
        if utils.is_next_token_one_of(['end'], iCurrent, lObjects):
            break
        iCurrent = concurrent_statement.detect(iCurrent, lObjects)

    if not utils.are_next_consecutive_tokens(['end', 'generate'], iCurrent, lObjects):
        iCurrent = utils.assign_next_token_required('end', token.end_keyword, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_if_not(';', token.alternative_label, iCurrent, lObjects)
        iCurrent = utils.assign_next_token_required(';', token.semicolon, iCurrent, lObjects)

    return iCurrent




from vsg.rules import token_indent

from vsg import token

lTokens = []
lTokens.append(token.case_statement.case_keyword)
lTokens.append(token.case_statement.end_keyword)
lTokens.append(token.case_statement_alternative.when_keyword)


class rule_001(token_indent):
    '''
    This rule checks the indent of **case**, **when**, and **end case** keywords.

    **Violation**

    .. code-block:: vhdl


       case data is

           when 0 =>
       when 1 =>
               when 3 =>

      end case;

    **Fix**

    .. code-block:: vhdl

      case data is

        when 0 =>
        when 1 =>
        when 3 =>

      end case;
    '''

    def __init__(self):
        token_indent.__init__(self, 'case', '001', lTokens)
